/* 30/01/97 10:25
 ******************************************************************************
 **
 **  MODULO : cliente.c
 **
 **  FUNCAO : rotinas do CLIENTE
 **
 ******************************************************************************
 *
 ******************************************************************************
 **
 **  Registro de Alteracoes/Versoes :
 **  ==============================
 **
 **  VERSAO   DATA    RESPONSAVEL   DESCRICAO
 **
 **  V1.00  18/06/96  Magoga        Criacao do arquivo
 **  V2.00  03/04/98  Borba         Inclusao novos comandos p/LCD, PINPAD,
 **                                 cartao magnetico e impressora.
 **  V2.01  2003/04/11, Tarcisio Kreutz, Perto S.A.
 **      - Inserted "Borba new commands" into Perto sources.
 **      - Conditional compilation:
 **  				+ PERTO_NOT_STANDARD: "Translates" the '=' command to the 'P' 
 **              command [alinha_e_le()] as changed by Qualita.
 **            + PERTO_NO_SERVER_DAEMON: The "server" code is linked to the
 **              "client" code, so that no server daemon process is needed.
 **      - The server now handles the "DTR off" mode of the new Pertochek.
 **      - Previous version stored client msg queue ids in static area.
 **        Than, if the same client (same pid == same key) allocated a new
 **        msg queue to replace a previous one, the server was not able to
 **        write the response to the client's queue.
 **        Fixed in two modes: 
 **         - PERTO_OLDCLT_COMPAT defined: The server gets the queue id each 
 **        time it sends a response. Client continues to use its PID as the 
 **        msg key.
 **         - PERTO_OLDCLT_COMPAT not defined: The client allocates its
 **        queue with IPC_PRIVATE and sends the msg queue ID to the server,
 **        instead of the msg queue KEY. 
 **      - TODO: IPC performance could be improved. No time for this now...
 **  V2.03  2004/02/09, Tarcisio Kreutz, Perto S.A.
 **      - Corrected bug at cad_cheques: The '2' command returned the 
 **        information shifted left 3 bytes.
 **  V2.04  2004/02/10, MEGS, Perto S.A.
 **      - Corrigido problema na rotina envia_conf. A mesma estava pegando os 
 **        dados da mensagem na posio errada. 
 **
 **  V2.05  ????/??/??, Flvio, Perto S.A. (redigido por MEGS)
 **      - Foram removidas as seguintes rotinas: pos_marc_benef, pos_marc,
 **        pos_marc_par, pos_marc_benef_par, envia_data, envia_num_maq,
 **        envia_hora, prg_num_maq e prg_data.
 **      - Acrescentado parametro "banco" na envia_conf.
 **      - Foram incluidas as seguintes rotinas: envia_ultimo_cmc7, 
 **        prog_lista_negra, imp_folha_parcial, imp_completo_chq_TB, 
 **        imp_parcial_chq_TB, esp_tecla_pchek, envia_mostra_com_lcd,
 **        alinha_e_le_cod_barras e envia_status_equip.
 **
 **  V2.06  2005/04/05, MEGS, Perto S.A.
 **      - Rotinas "imp_folha" e "imp_folha_parcial" foram otimizadas para no
 **        completar com brancos.
 **      - Rotina "le_cartao" passa a aceitar os outros tipos de criptografia.
 **      - Rotina "estado_perto" foi revisada. Novos erros foram incluidos.
 **      - Rotina "prog_lista_negra" foi corrigida.
 **      - Rotina "chama_servidor" teve alguns retornos considerados como OK.
 **
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  COMENTARIOS  :                                                           *
 **                                                                           *
 **                                                                           *
 **  ATENCAO :                                                                *
 **  =======                                                                  *
 **                                                                           *
 **                                                                           *
 ******************************************************************************
 */

/*#######################  I N C L U D E S  ##################################*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>

#include "perto.h"

extern int errno;

/*#######################  C O N S T A N T E S  ##############################*/

#define NDIG        8
#define MAXOPEN     20

/*#######################  P R O T O T I P O S  ##############################*/

#ifdef ANSI
int pcheck_get_config(char *nome, char *arq, st_serverinfo_t *info, int *err);
static int chama_servidor(MESSAGE *);
#ifndef PERTO_NO_SERVER_DAEMON
static int open_queue(long, int);
static int send(int, MESSAGE *, int);
static int receive(int, MESSAGE *, int);
static int remove_queue(long);
#endif /* PERTO_NO_SERVER_DAEMON */
static void pnum(char *, int, int, int);
static void anum(int *, char *, int);
#else
int pcheck_get_config();
static int chama_servidor();
#ifndef PERTO_NO_SERVER_DAEMON
static int open_queue();
static int send();
static int receive();
static int remove_queue();
#endif /* PERTO_NO_SERVER_DAEMON */
static int pnum();
static int anum();
#endif

#ifdef PERTO_NO_SERVER_DAEMON
#ifdef ANSI
extern int inic_stxetx(char *, int, int);
extern int stop_stxetx(void);
extern void talk_to_pertochek(MESSAGE *);
#else
extern int inic_stxetx();
extern int stop_stxetx();
extern void talk_to_pertochek();
#endif
#endif /* PERTO_NO_SERVER_DAEMON */

/*#######################  V A R I A V E I S  ################################*/

/*=======================  L o c a i s  ======================================*/

static int erro_perto = P_TERMINO_OK;

#ifndef PERTO_NO_SERVER_DAEMON
static long chave_servidor = -1L;
static long chave_cliente = -1L;
#endif /* PERTO_NO_SERVER_DAEMON */

#ifndef PERTO_NOT_STANDARD
static char arq_conf[] = { "arq_conf.dat" };
#else
#ifdef DEBUG
static char arq_conf[] = { "./arq_conf.dat" };
#else
static char arq_conf[] = { "/u/cartlink/arq_conf.dat" };
#endif
#endif /* PERTO_NOT_STANDARD */

/*=======================  P u b l i c a s  ==================================*/

/*=======================  E x t e r n a s  ==================================*/

/*#######################  C O M A N D O S  ##################################*/

#ifdef ANSI
int inic_cliente(char *nome_perto)
#else
int inic_cliente(nome_perto)
     char *nome_perto;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;
	st_serverinfo_t srvcfg;

	erro_perto = P_TERMINO_OK;

   if (! pcheck_get_config(nome_perto, arq_conf, &srvcfg, &erro_perto))
   {
   	return (FALSE);
   }
#ifndef PERTO_NO_SERVER_DAEMON
	chave_servidor = srvcfg.msgkey;
   chave_cliente = getpid();
#else
   if (! inic_stxetx(srvcfg.device, srvcfg.baud,
				(srvcfg.pchekcaps & PCHEK_CAPS_DTROFF)))
   {
		erro_perto = P_ERRO_INIT_SERIAL;
#ifndef PERTO_NOT_STANDARD
		return (FALSE);
#else
#ifndef CLIENT_NO_MESSAGES
      fprintf(stderr, "ERRO: comunicacoes\n");
#endif /* CLIENT_NO_MESSAGES */
      exit(1);
#endif /* PERTO_NOT_STANDARD */
   }
#endif /* PERTO_NO_SERVER_DAEMON */

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_VERSAO;

   return (chama_servidor(&m));
}

#ifdef ANSI
int fim_cliente(void)
#else
int fim_cliente()
#endif
{
#ifndef PERTO_NO_SERVER_DAEMON
   return (remove_queue(chave_cliente));
#else
	stop_stxetx();
#ifndef PERTO_NOT_STANDARD
   return (TRUE);
#else
   return (0);
#endif /* PERTO_NOT_STANDARD */
#endif /* PERTO_NO_SERVER_DAEMON */
}

#ifdef ANSI
int prg_data_chk(char *data)
#else
int prg_data_chk(data)
     char *data;
#endif
{
   MESSAGE m;
   struct p_cmd4 *ptr;

   ptr = (struct p_cmd4 *)&m.dados[0];
   ptr->cmd = P_PRG_DATA_CHK;

   if (strlen(data) != 6)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->data_hora, data, 6);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_verso(char *linha1, char *linha2)
#else
int imp_verso(linha1, linha2)
     char *linha1;
     char *linha2;
#endif
{
   MESSAGE m;
   struct p_cmd11 *ptr;
   int tam;

   ptr = (struct p_cmd11 *)&m.dados[0];
   ptr->cmd = P_IMP_VERSO;

   tam = strlen(linha1);

   if (tam > 60)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->linha1, ' ', 60);
   memcpy(ptr->linha1, linha1, tam);

   tam = strlen(linha2);

   if (tam > 60)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->linha2, ' ', 60);
   memcpy(ptr->linha2, linha2, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_cidade(char *cidade)
#else
int prg_cidade(cidade)
     char *cidade;
#endif
{
   MESSAGE m;
   struct p_cmd2 *ptr;
   int tam;

   ptr = (struct p_cmd2 *)&m.dados[0];
   ptr->cmd = P_PRG_CIDADE;

   tam = strlen(cidade);

   if (tam > 20)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->cidade, ' ', 20);
   memcpy(ptr->cidade, cidade, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_completo(char modo, char *valor, char *banco)
#else
int imp_completo(modo, valor, banco)
     char modo;
     char *valor;
     char *banco;
#endif
{
   MESSAGE m;
   struct p_cmd1 *ptr;
   int tam;

   ptr = (struct p_cmd1 *)&m.dados[0];
   ptr->cmd = P_IMP_COMPLETO;

   ptr->modo = modo;

   tam = strlen(valor);

   if (tam > 12 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->valor, '0', 12);
   memcpy(ptr->valor + 12 - tam, valor, tam);

   tam = strlen(banco);

   if (tam != 3)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->banco, banco, 3);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_parcial(char modo, char *valor, char *banco)
#else
int imp_parcial(modo, valor, banco)
     char modo;
     char *valor;
     char *banco;
#endif
{
   MESSAGE m;
   struct p_cmd1 *ptr;
   int tam;

   ptr = (struct p_cmd1 *)&m.dados[0];
   ptr->cmd = P_IMP_PARCIAL;

   ptr->modo = modo;

   tam = strlen(valor);

   if (tam > 12 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->valor, '0', 12);
   memcpy(ptr->valor + 12 - tam, valor, tam);

   tam = strlen(banco);

   if (tam != 3)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->banco, banco, 3);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_benef(char *nome)
#else
int prg_benef(nome)
     char *nome;
#endif
{
   MESSAGE m;
   struct p_cmd3 *ptr;
   int tam;

   ptr = (struct p_cmd3 *)&m.dados[0];
   ptr->cmd = P_PRG_BENEFIC;

   tam = strlen(nome);

   if (tam > 50)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->nome, ' ', 50);
   memcpy(ptr->nome, nome, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_copia(char *finalidade1, char *finalidade2, char *banco, char *num_chk)
#else
int imp_copia(finalidade1, finalidade2, banco, num_chk)
     char *finalidade1;
     char *finalidade2;
     char *banco;
     char *num_chk;
#endif
{
   MESSAGE m;
   struct p_cmd7 *ptr;
   int tam;

   ptr = (struct p_cmd7 *)&m.dados[0];
   ptr->cmd = P_IMP_COPIA;

   tam = strlen(finalidade1);

   if (tam > 78)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->finalidade1, ' ', 78);
   memcpy(ptr->finalidade1, finalidade1, tam);

   tam = strlen(finalidade2);

   if (tam > 78)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->finalidade2, ' ', 78);
   memcpy(ptr->finalidade2, finalidade2, tam);

   tam = strlen(banco);

   if (tam > 20)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->banco, ' ', 20);
   memcpy(ptr->banco, banco, tam);

   tam = strlen(num_chk);

   if (tam > 6 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->num_chk, '0', 6);
   memcpy(ptr->num_chk + 6 - tam, num_chk, tam);

   return (chama_servidor(&m));
}


#ifdef ANSI
int envia_benef(char *nome)
#else
int envia_benef(nome)
     char *nome;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_BENEF;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   memset(nome, ' ', 50);
   memcpy(nome, &m.dados[1], 50);
   nome[50] = 0;

   return (TRUE);
}

#ifdef ANSI
int envia_cidade(char *cidade)
#else
int envia_cidade(cidade)
     char *cidade;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_CIDADE;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   memset(cidade, ' ', 20);
   memcpy(cidade, &m.dados[1], 20);
   cidade[20] = 0;

   return (TRUE);
}

#ifndef PERTO_NOT_STANDARD

#ifdef ANSI
int alinha_e_le(char *banco, char *agencia, char *conta, char *num_chk)
#else
int alinha_e_le(banco, agencia, conta, num_chk)
     char *banco;
     char *agencia;
     char *conta;
     char *num_chk;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ALINHA_E_LE;

   if (chama_servidor(&m) == FALSE)
	{
      /*erro_perto = P_ERRO_CMC;*/
      return (FALSE);
	}

   if (m.estado == P_TERMINO_OK)
   {
      memcpy(banco, &m.dados[4], 3);
      banco[3] = 0;
      memcpy(agencia, &m.dados[7], 4);
      agencia[4] = 0;
      memcpy(conta, &m.dados[11], 10);
      conta[10] = 0;
      memcpy(num_chk, &m.dados[21], 6);
      num_chk[6] = 0;
      return (TRUE);
   }

   /*erro_perto = P_ERRO_CMC;*/
   return (FALSE);
}

#else /* PERTO_NOT_STANDARD */

#ifdef ANSI
void limpa_valor(char *valstr)
#else
void limpa_valor(valstr)
     char *valstr;
#endif
{
   char inddest, indorig;

   inddest = indorig = 0;
   while (valstr[indorig] != 0)
   {
      if ((valstr[indorig] >= '0') && (valstr[indorig] <= '9'))
      {
         valstr[inddest] = valstr[indorig];
         inddest++;
      }
      indorig++;
   }
   valstr[inddest] = 0;
}

/*----------------------------------------------------------------------------* 
 | Funcao DigCMC7 - Autor: FASC
 *----------------------------------------------------------------------------
 | Descricao:  Calcula digito verificador do CMC7.
 | Param (IN): szOrig -> string na qual sera calculado o digito verificador
 | Retorno:    Digito verificador ou 'E' caso haja algum erro
 *----------------------------------------------------------------------------*/
#ifdef ANSI
char DigCMC7(char *szOrig)
#else
char DigCMC7(szOrig)
	char *szOrig;
#endif
{
   unsigned char ibModulo10, ibAcumulador, ibAcumAux, ibInd, ibValor;
   char szStrAux[256];

   ibValor = 0;
   ibAcumulador = 0;
   ibModulo10 = 0x2;

   ibInd = strlen(szOrig);
   strcpy(szStrAux, szOrig);
   while (ibInd >= 1)
   {
      ibInd--;
      ibValor = szStrAux[ibInd] - '0';
      if (ibValor > 9)
         return 'E';
      ibValor *= ibModulo10;

      if (ibValor >= 10)
         if (ibValor < 0x10)
         {
            ibValor &= 0x7;
            ibValor--;
         }
         else if (ibValor == 0x10)
            ibValor = 0x7;
         else
            ibValor = 0x9;

      ibAcumulador += ibValor;
      if (ibModulo10 == 0x2)
         ibModulo10 = 0x1;
      else
         ibModulo10 = 0x2;
   }

   ibAcumAux = ibAcumulador;
   do
   {
      ibValor = ibAcumulador % 10;
      if (ibValor != 0)
         ibAcumulador++;
   } while (ibValor != 0);

   return ibAcumulador - ibAcumAux + '0';
}

#ifdef ANSI
int alinha_e_le(char *banco, char *agencia, char *conta, char *num_chk)
#else
int alinha_e_le(banco, agencia, conta, num_chk)
     char *banco;
     char *agencia;
     char *conta;
     char *num_chk;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;
   char desloc;
   char dig1, dig2, dig3;
   char *psCodCMC7, szStrAux[11];

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ALINHA_E_LE;

   if (chama_servidor(&m) == FALSE)
   {
      return (FALSE);
   }

   if (m.estado == P_TERMINO_OK)
   {
      /* As vezes falha na leitura do CMC7 e nao vem o 'A' */
      if (m.dados[4] == 'A')
         desloc = 5;
      else
         desloc = 4;
      psCodCMC7 = m.dados + desloc;

      memcpy(banco, &m.dados[desloc], 3);
      banco[3] = 0;
      desloc += 3;
      memcpy(agencia, &m.dados[desloc], 4);
      agencia[4] = 0;
      desloc += 4 + 2;
      desloc += 3;
      memcpy(num_chk, &m.dados[desloc], 6);
      num_chk[6] = 0;
      desloc += 6 + 3;
      memcpy(conta, &m.dados[desloc], 10);
      conta[10] = 0;

      limpa_valor(banco);
      limpa_valor(agencia);
      limpa_valor(conta);
      limpa_valor(num_chk);
      if ((strlen(banco) < 3) ||
          (strlen(agencia) < 4) ||
          (strlen(conta) < 10) || (strlen(num_chk) < 6))
      {
         erro_perto = P_ERRO_CMC;
         return (FALSE);
      }

      /* Testa para ver se nao eh o cheque exemplo da perto,
       * pois ele (na versao antiga) tem o digito calculado 
		 * de forma errada */
      if (strcmp(banco, "000"))
      {
         strncpy(szStrAux, psCodCMC7, 7);
         szStrAux[7] = 0;
         dig1 = DigCMC7(szStrAux);
         strncpy(szStrAux, psCodCMC7 + 9, 10);
         szStrAux[10] = 0;
         dig2 = DigCMC7(szStrAux);
         strncpy(szStrAux, psCodCMC7 + 21, 10);
         szStrAux[10] = 0;
         dig3 = DigCMC7(szStrAux);
         if (dig1 != psCodCMC7[20] ||
             dig2 != psCodCMC7[7] || dig3 != psCodCMC7[31])
         {
            erro_perto = P_ERRO_CMC;
            return FALSE;
         }
      }
      return (TRUE);
   }
   else
   {
      erro_perto = P_ERRO_CMC;
      return (FALSE);
   }
}

#endif /* PERTO_NOT_STANDARD */

#ifdef ANSI
int expulsa_cheque(void)
#else
int expulsa_cheque()
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_EXPULSA_CHEQUE;

   return (chama_servidor(&m));
}

/*
 * Comandos para cad_cheques:
 *
 *          '0' - descadastra cheque
 *          '1' - cadastra cheque
 *          '2' - informa cadastro
 */

#ifdef ANSI
int cad_cheques(char comando,
                char *banco,
                int *lin_val_num, int *col_val_num, int *lin_ano, int *col_ano,
                int *lin_val_ext1, int *lin_val_ext2, int *col_val_ext,
                int *lin_nome, int *col_data, int *col_total)
#else
int cad_cheques(comando, banco, lin_val_num, col_val_num, lin_ano, col_ano,
                lin_val_ext1, lin_val_ext2, col_val_ext, lin_nome, col_data,
                col_total)
     char comando;
     char *banco;
     int *lin_val_num;
     int *col_val_num;
     int *lin_ano;
     int *col_ano;
     int *lin_val_ext1;
     int *lin_val_ext2;
     int *col_val_ext;
     int *lin_nome;
     int *col_data;
     int *col_total;
#endif
{
   MESSAGE m;
   struct p_cmd10 *ptr;

   ptr = (struct p_cmd10 *)&m.dados[0];

   memset((char *)ptr, '0', sizeof(struct p_cmd10));

   ptr->cmd = P_CAD_CHEQUES;
   ptr->cmd2 = comando;

   memcpy(ptr->banco, banco, 3);

   if (comando == '1')
   {
      pnum(ptr->col_val_num, *col_val_num, 3, 10);
      pnum(ptr->col_val_ext, *col_val_ext, 3, 10);
      pnum(ptr->col_ano, *col_ano, 3, 10);
      pnum(ptr->col_data, *col_data, 3, 10);
      pnum(ptr->lin_val_num, *lin_val_num, 3, 10);
      pnum(ptr->lin_val_ext1, *lin_val_ext1, 3, 10);
      pnum(ptr->lin_val_ext2, *lin_val_ext2, 3, 10);
      pnum(ptr->lin_nome, *lin_nome, 3, 10);
      pnum(ptr->lin_ano, *lin_ano, 3, 10);
      pnum(ptr->col_total, *col_total, 3, 10);
   }

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (comando == '2')
   {
   	struct p_cmd10_rep *ptrrep = (struct p_cmd10_rep *)ptr;

		if (memcmp(ptrrep->resultado, "000", 3))
		  return FALSE;

      anum(col_val_num, ptrrep->col_val_num, 3);
      anum(col_val_ext, ptrrep->col_val_ext, 3);
      anum(col_ano, ptrrep->col_ano, 3);
      anum(col_data, ptrrep->col_data, 3);
      anum(lin_val_num, ptrrep->lin_val_num, 3);
      anum(lin_val_ext1, ptrrep->lin_val_ext1, 3);
      anum(lin_val_ext2, ptrrep->lin_val_ext2, 3);
      anum(lin_nome, ptrrep->lin_nome, 3);
      anum(lin_ano, ptrrep->lin_ano, 3);
      anum(col_total, ptrrep->col_total, 3);
   }

   return (TRUE);
}

#ifdef ANSI
int envia_conf(char *data, char *banco, char *c1, char *c2, char *c3, char *c4,
               char *c5, char *num_chk)
#else
int envia_conf(data, banco, c1, c2, c3, c4, c5, num_chk)
   char *data;
   char *banco;
   char *c1;
   char *c2;
   char *c3;
   char *c4;
   char *c5;
   char *num_chk;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_CONF;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   memcpy(data, &m.dados[1], 6);
   data[6] = 0;

   memcpy(banco, &m.dados[7], 3);
   banco[3] = 0;

   c1[0] = m.dados[10];
   c1[1] = 0;
   c2[0] = m.dados[11];
   c2[1] = 0;
   c3[0] = m.dados[12];
   c3[1] = 0;
   c4[0] = m.dados[13];
   c4[1] = 0;
   c5[0] = m.dados[14];
   c5[1] = 0;
   
   memcpy(num_chk, &m.dados[15], 3);
   num_chk[3] = 0;

   return (TRUE);
}

#ifdef ANSI
int prg_modo(char modo)
#else
int prg_modo(modo)
     char modo;
#endif
{
   MESSAGE m;
   struct p_cmd6 *ptr;

   ptr = (struct p_cmd6 *)&m.dados[0];
   ptr->cmd = P_PRG_MODO;
   ptr->modo = modo;

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_colunas(char *texto1, char *texto2, char *texto3)
#else
int prg_colunas(texto1, texto2, texto3)
     char *texto1;
     char *texto2;
     char *texto3;
#endif
{
   MESSAGE m;
   struct p_cmd14 *ptr;
   int tam;

   ptr = (struct p_cmd14 *)&m.dados[0];
   ptr->cmd = P_PRG_COLUNAS;

   tam = strlen(texto1);

   if (tam > 15)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->texto1, ' ', 15);
   memcpy(ptr->texto1, texto1, tam);

   tam = strlen(texto2);

   if (tam > 15)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->texto2, ' ', 15);
   memcpy(ptr->texto2, texto2, tam);

   tam = strlen(texto3);

   if (tam > 15)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->texto3, ' ', 15);
   memcpy(ptr->texto3, texto3, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_moeda(char *moeda)
#else
int prg_moeda(moeda)
     char *moeda;
#endif
{
   MESSAGE m;
   struct p_cmd8 *ptr;
   int tam;

   ptr = (struct p_cmd8 *)&m.dados[0];
   ptr->cmd = P_PRG_MOEDA;

   tam = strlen(moeda);

   if (tam > 20 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->moeda, ' ', 20);
   memcpy(ptr->moeda, moeda, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_telefone(char *ddd, char *telefone)
#else
int prg_telefone(ddd, telefone)
     char *ddd;
     char *telefone;
#endif
{
   MESSAGE m;
   struct p_cmd12 *ptr;

   ptr = (struct p_cmd12 *)&m.dados[0];
   ptr->cmd = P_PRG_TELEFONE;

   if (strlen(ddd) != 4)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->ddd, ddd, 4);

   if (strlen(telefone) != 10)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->telefone, telefone, 10);

   return (chama_servidor(&m));
}

#ifdef ANSI
int envia_versao(char *versao)
#else
int envia_versao(versao)
     char *versao;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_VERSAO;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   memcpy(versao, &m.dados[1], 17);
   versao[17] = 0;

   return (TRUE);
}

#ifdef ANSI
int prg_cheque_adm(char *cidade, char *nome)
#else
int prg_cheque_adm(cidade, nome)
     char *cidade;
     char *nome;
#endif
{
   MESSAGE m;
   struct p_cmd15 *ptr;
   int tam;

   ptr = (struct p_cmd15 *)&m.dados[0];
   ptr->cmd = P_PRG_CHEQUE_ADM;

   tam = strlen(cidade);

   if (tam > 20)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->cidade, ' ', 20);
   memcpy(ptr->cidade, cidade, tam);

   tam = strlen(nome);

   if (tam > 50)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->nome, ' ', 50);
   memcpy(ptr->nome, nome, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_cheque_adm(char modo, char *valor, char *banco)
#else
int imp_cheque_adm(modo, valor, banco)
     char modo;
     char *valor;
     char *banco;
#endif
{
   MESSAGE m;
   struct p_cmd1 *ptr;
   int tam;

   ptr = (struct p_cmd1 *)&m.dados[0];
   ptr->cmd = P_IMP_CHEQUE_ADM;

   ptr->modo = modo;

   tam = strlen(valor);

   if (tam > 12 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->valor, '0', 12);
   memcpy(ptr->valor + 12 - tam, valor, tam);

   tam = strlen(banco);

   if (tam != 3)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->banco, banco, 3);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_bom_para(char *texto)
#else
int prg_bom_para(texto)
     char *texto;
#endif
{
   MESSAGE m;
   struct p_cmd18 *ptr;
   int tam;

   ptr = (struct p_cmd18 *)&m.dados[0];
   ptr->cmd = P_BOM_PARA;

   tam = strlen(texto);

   if (tam > 20)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->linha, ' ', 20);
   memcpy(ptr->linha, texto, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int prg_campo_extra(char *texto)
#else
int prg_campo_extra(texto)
     char *texto;
#endif
{
   MESSAGE m;
   struct p_cmd18 *ptr;
   int tam;

   ptr = (struct p_cmd18 *)&m.dados[0];
   ptr->cmd = P_CAMPO_EXTRA;

   tam = strlen(texto);

   if (tam > 20)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->linha, ' ', 20);
   memcpy(ptr->linha, texto, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_folha_int(char *linha1, char *linha2, char *linha3, char *linha4,
              char *linha5, char *linha6, char *linha7, char *linha8,
              char *linha9, char *linha10, char *linha11, char *linha12,
              char *linha13, char *linha14, char chTipo)
#else
int imp_folha_int(linha1, linha2, linha3, linha4,
              linha5, linha6, linha7, linha8,
              linha9, linha10, linha11, linha12, linha13, linha14, chTipo)
     char *linha1;
     char *linha2;
     char *linha3;
     char *linha4;
     char *linha5;
     char *linha6;
     char *linha7;
     char *linha8;
     char *linha9;
     char *linha10;
     char *linha11;
     char *linha12;
     char *linha13;
     char *linha14;
     char chTipo;
#endif
{
   MESSAGE m;
   struct p_cmd19 *ptr;
   int tam, n, x;
   char *plinha[14];

   /* inicializa as variaveis */
   ptr = (struct p_cmd19 *)&m.dados[0];
   plinha[0] = linha1; plinha[1] = linha2; plinha[2] = linha3;
   plinha[3] = linha4; plinha[4] = linha5; plinha[5] = linha6;
   plinha[6] = linha7; plinha[7] = linha8; plinha[8] = linha9;
   plinha[9] = linha10; plinha[10] = linha11; plinha[11] = linha12;
   plinha[12] = linha13; plinha[13] = linha14;
   n = 0;
   
   /* Coloca o comando na estrutura */
   ptr->cmd = chTipo;

   /* transfere as linhas para o buffer */
   for (x = 0; x < 14; x++) {
      /* Se tamanho incorreto */
      tam = strlen(plinha[x]);
      if (tam > 78) {
         erro_perto = P_ERRO_PARAMETRO;
         return (FALSE);
      }
   
      /* copia a linha para a estrutura */
      memcpy(ptr->linhas + n, plinha[x], tam); n+= tam;
      ptr->linhas[n++] = 0xFF;
   }

   /* programa o tamanho do buffer */
   m.tam = n + 1;

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_folha(char *linha1, char *linha2, char *linha3, char *linha4,
              char *linha5, char *linha6, char *linha7, char *linha8,
              char *linha9, char *linha10, char *linha11, char *linha12,
              char *linha13, char *linha14)
#else
int imp_folha(linha1, linha2, linha3, linha4,
              linha5, linha6, linha7, linha8,
              linha9, linha10, linha11, linha12, linha13, linha14)
     char *linha1;
     char *linha2;
     char *linha3;
     char *linha4;
     char *linha5;
     char *linha6;
     char *linha7;
     char *linha8;
     char *linha9;
     char *linha10;
     char *linha11;
     char *linha12;
     char *linha13;
     char *linha14;
#endif
{
   return imp_folha_int(linha1, linha2, linha3, linha4, linha5, linha6, linha7,
         linha8, linha9, linha10, linha11, linha12, linha13, linha14,
         P_IMP_FOLHA);
}

#ifdef ANSI
int imp_folha_parcial(char *linha1, char *linha2, char *linha3, char *linha4,
                      char *linha5, char *linha6, char *linha7, char *linha8,
                      char *linha9, char *linha10, char *linha11, char *linha12,
                      char *linha13, char *linha14)
#else
int imp_folha_parcial(linha1, linha2, linha3, linha4, linha5, linha6, linha7,
                      linha8, linha9, linha10, linha11, linha12, linha13,
                      linha14)
   char *linha1;
   char *linha2;
   char *linha3;
   char *linha4;
   char *linha5;
   char *linha6;
   char *linha7;
   char *linha8;
   char *linha9;
   char *linha10;
   char *linha11;
   char *linha12;
   char *linha13;
   char *linha14;
#endif
{
   return imp_folha_int(linha1, linha2, linha3, linha4, linha5, linha6, linha7,
         linha8, linha9, linha10, linha11, linha12, linha13, linha14,
         P_IMP_FOLHA_PARCIAL);
}

#ifdef ANSI
int prg_configuracao2(char p2, char p3, char p4, char p5)
#else
int prg_configuracao2(p2, p3, p4, p5)
     char p2;
     char p3;
     char p4;
     char p5;
#endif
{
   MESSAGE m;
   struct p_cmd20 *ptr;
   int tam;

   if (p5 != '0')
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   ptr = (struct p_cmd20 *)&m.dados[0];
   ptr->cmd = P_PRG_MODO2;

   ptr->p2 = p2;
   ptr->p3 = p3;
   ptr->p4 = p4;
   ptr->p5 = p5;

   return (chama_servidor(&m));
}

#ifdef ANSI
int alinha_le_envia(char *cmc7)
#else
int alinha_le_envia(cmc7)
     char *cmc7;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ALINHA_LE_ENVIA;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (m.estado == P_TERMINO_OK)
   {
      memset(cmc7, ' ', 80);
      memcpy(cmc7, &m.dados[4], m.tam - 4);
      cmc7[m.tam - 4] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}

#ifdef ANSI
int envia_ultimo_cmc7(char *cmc7)
#else
int envia_ultimo_cmc7(cmc7)
     char *cmc7;
#endif
{
   MESSAGE m;
   struct p_cmd0 *ptr;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_ULTIMO_CMC7;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (m.estado == P_TERMINO_OK) {
      memcpy(cmc7, &m.dados[1], m.tam - 1);
      cmc7[m.tam-1] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}

#ifdef ANSI
int prg_chave_cripto(char *chave)
#else
int prg_chave_cripto(chave)
     char *chave;
#endif
{
   MESSAGE m;
   struct p_cmd26 *ptr;
   int tam;

   ptr = (struct p_cmd26 *)&m.dados[0];
   ptr->cmd = P_PRG_CHAVE_CRIPTO;

   if (strlen(chave) != 8)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->chave, chave, 8);

   return (chama_servidor(&m));
}


#ifdef ANSI
int le_cartao(char *seg, char cripto, char *trilha2)
#else
int le_cartao(seg, cripto, trilha2)
     char *seg;
     char cripto;
     char *trilha2;
#endif
{
   MESSAGE m;
   struct p_cmd21 *ptr;
   int tam;

   tam = strlen(seg);
   if ((tam != 3) || ((seg[0] - 0x30) * 100 + (seg[1] - 0x30) * 10 + 
            (seg[2] - 0x30) > 255)) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   ptr = (struct p_cmd21 *)&m.dados[0];
   ptr->cmd = P_LE_CARTAO;
   memcpy(ptr->seg, seg, 3);

   if ((cripto != '0') && (cripto != '1') && (cripto != '2') && 
         (cripto != '3') && (cripto != '4')) {
      erro_perto = P_ERRO_PARAMETRO;
      return FALSE;
   }

   ptr->x = cripto;
   m.tam = 5;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if ((m.estado == P_TERMINO_OK) ||
       (m.estado == P_ERRO_LEITURA_TRILHA1) ||
       (m.estado == P_ERRO_LEITURA_TRILHA2) ||
       (m.estado == P_ERRO_LEITURA_TRILHA3)) {
      memcpy(trilha2, &m.dados[4], m.tam - 4);
      trilha2[m.tam - 4] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}

#ifdef ANSI
int envia_com_imp(char *buf)
#else
int envia_com_imp(buf)
     char *buf;
#endif
{
   MESSAGE m;
   struct p_cmd22 *ptr;
   int tam;

   ptr = (struct p_cmd22 *)&m.dados[0];
   ptr->cmd = P_ENVIA_COM_IMP;

   tam = strlen(buf);
   if (tam > 40) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memset(ptr->buf, ' ', 40);
   memcpy(ptr->buf, buf, tam);

   return (chama_servidor(&m));
}

#ifdef ANSI
int espera_tec_pinpad(char *seg, char *tecla)
#else
int espera_tec_pinpad(seg, tecla)
     char *seg;
     char *tecla;
#endif
{
   MESSAGE m;
   struct p_cmd23 *ptr;
   int tam;

   ptr = (struct p_cmd23 *)&m.dados[0];
   ptr->cmd = P_ESPERA_TEC_PINPAD;

   tam = strlen(seg);

   if ((tam != 3) ||
       ((seg[0] - 0x30) * 100 + (seg[1] - 0x30) * 10 + (seg[2] - 0x30) > 255))
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->seg, seg, 3);

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (m.estado == P_TERMINO_OK)
   {
      memcpy(tecla, &m.dados[4], m.tam - 4);
      tecla[m.tam - 4] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}


#ifdef ANSI
int envia_com_lcd(char cmd2, char *poscur, char *mensagem)
#else
int envia_com_lcd(cmd2, poscur, mensagem)
     char cmd2;
     char *poscur;
     char *mensagem;
#endif
{
   MESSAGE m;
   struct p_cmd24 *ptr;
   int tam;

   ptr = (struct p_cmd24 *)&m.dados[0];
   ptr->cmd = P_ENVIA_COM_LCD;

   if (cmd2 == '0' || cmd2 == '1' || cmd2 == '5' || cmd2 == '6') {
      ptr->cmd2 = cmd2;
      tam = 2;
   }
   else if (cmd2 == '2' || cmd2 == '3' || cmd2 == '4' ||
          cmd2 == '7' || cmd2 == '8' || cmd2 == '9') {
      tam = strlen(poscur);
      if (tam != 2) {
         erro_perto = P_ERRO_PARAMETRO;
         return (FALSE);
      }
      
      tam = strlen(mensagem);
      if (tam > 16) {
         erro_perto = P_ERRO_PARAMETRO;
         return (FALSE);
      }

      ptr->cmd2 = cmd2;
      memcpy(ptr->poscur, poscur, 2);
      memset(ptr->mensagem, ' ', 16);
      memcpy(ptr->mensagem, mensagem, tam);
      tam = tam + 4;
   }
   else {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   m.tam = tam;
   return (chama_servidor(&m));
}

#ifdef ANSI
int envia_com_pinpad(char *seg, char pb, char pp, char pr, char pe,
                     char *comando, char *resposta)
#else
int envia_com_pinpad(seg, pb, pp, pr, pe, comando, resposta)
     char *seg;
     char pb;
     char pp;
     char pr;
     char pe;
     char *comando;
     char *resposta;
#endif
{
   MESSAGE m;
   struct p_cmd25 *ptr;
   int tam;

   ptr = (struct p_cmd25 *)&m.dados[0];
   ptr->cmd = P_ENVIA_COM_PINPAD;

   tam = strlen(seg);
   if ((tam != 3) || ((seg[0] - 0x30) * 100 + (seg[1] - 0x30) * 10 + 
            (seg[2] - 0x30) > 255)) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->seg, seg, 3);

   tam = strlen(comando);
   if (tam > 95) {
      erro_perto = P_ERRO_TAMANHO;
      return (FALSE);
   }
   memcpy(ptr->comando, comando, tam);

   ptr->fs = CTE_FS;

   if (!(pb == '0' || pb == '1' || pb == '2' || pb == '3' || pb == '4'))
   {
      erro_perto = P_ERRO_DADOS;
      return (FALSE);
   }

   ptr->pb = pb;

   if (!(pp == '0' || pp == '1'))
   {
      erro_perto = P_ERRO_DADOS;
      return (FALSE);
   }

   ptr->pp = pp;

   if (!(pr == '0' || pr == '1'))
   {
      erro_perto = P_ERRO_DADOS;
      return (FALSE);
   }

   ptr->pr = pr;

   if (!(pe == '0' || pe == '1'))
   {
      erro_perto = P_ERRO_DADOS;
      return (FALSE);
   }

   ptr->pe = pe;

   m.tam = tam + 9;

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (m.estado == P_TERMINO_OK)
   {
      memcpy(resposta, &m.dados[4], m.tam - 4);
      resposta[m.tam - 4] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}


/*
 * Comandos para prog_lista_negra:
 *
 *          '0' - exclui cheque
 *          '1' - inclui cheque
 *          '2' - consulta cheque
 */

#ifdef ANSI
int prog_lista_negra(char opcao, char *banco, char *agencia, char *conta)
#else
int prog_lista_negra(opcao, banco, agencia, conta)
   char opcao;
   char *banco;
   char *agencia;
   char *conta;
#endif
{
   MESSAGE m;
   struct p_cmd27 *ptr;
   int tam;

   /* preenche a estrutura com '0' */
   ptr = (struct p_cmd27 *)&m.dados[0];
   memset((char *)ptr, '0', sizeof(struct p_cmd27));

   /* coloca o comando */
   ptr->cmd = P_LISTA_NEGRA;
   ptr->opcao = opcao;

   /* banco */
   tam = strlen(banco);
   if (tam > 3) {
      erro_perto = P_ERRO_PARAMETRO;
      return FALSE;
   }
   memcpy(ptr->banco + 3 - tam, banco, tam);
   
   /* agencia */
   tam = strlen(agencia);
   if (tam > 4) {
      erro_perto = P_ERRO_PARAMETRO;
      return FALSE;
   }
   memcpy(ptr->agencia + 4 - tam, agencia, tam);
   
   /* conta */
   tam = strlen(conta);
   if (tam > 10) {
      erro_perto = P_ERRO_PARAMETRO;
      return FALSE;
   }
   memcpy(ptr->conta + 10 - tam, conta, tam);

   /* envia o comando */
   return chama_servidor(&m);
}

#ifdef ANSI
int imp_completo_chq_TB(char modo, char *valor, char *banco,
                        char *banco_dest, char *agencia_dest, char *conta_dest)
#else
int imp_completo_chq_TB(modo, valor, banco, banco_dest, agencia_dest, conta_dest)
   char modo;
   char *valor;
   char *banco;
   char *banco_dest;
   char *agencia_dest;
   char *conta_dest;
#endif
{
   MESSAGE m;
   struct p_cmd28 *ptr;
   int tam;

   ptr = (struct p_cmd28 *)&m.dados[0];
   
   ptr->cmd = P_IMP_COMPLETO_CHQ_TB;
   ptr->modo = modo;

   tam = strlen(valor);
   if (tam > 12 || !tam)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memset(ptr->valor, '0', 12);
   memcpy(ptr->valor + 12 - tam, valor, tam);
   memcpy(ptr->banco, banco, 3);

   tam = strlen(banco);
   if (tam != 3)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->banco, banco, 3);

   tam = strlen(banco_dest);
   if (tam != 3)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->banco_dest, banco_dest, 3);

   tam = strlen(agencia_dest);
   if (tam != 4)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->agencia_dest, agencia_dest, 4);

   tam = strlen(conta_dest);
   if (tam != 10)
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->conta_dest, conta_dest, 10);

   return (chama_servidor(&m));
}

#ifdef ANSI
int imp_parcial_chq_TB(char modo, char *valor, char *banco,
                        char *banco_dest, char *agencia_dest, char *conta_dest)
#else
int imp_parcial_chq_TB(modo, valor, banco, banco_dest, agencia_dest, conta_dest)
   char modo;
   char *valor;
   char *banco;
   char *banco_dest;
   char *agencia_dest;
   char *conta_dest;
#endif
{
   MESSAGE m;
   struct p_cmd28 *ptr;
   int tam;

   ptr = (struct p_cmd28 *)&m.dados[0];
   
   ptr->cmd = P_IMP_PARCIAL_CHQ_TB;
   ptr->modo = modo;

   tam = strlen(valor);
   if (tam > 12 || !tam) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memset(ptr->valor, '0', 12);
   memcpy(ptr->valor + 12 - tam, valor, tam);
   memcpy(ptr->banco, banco, 3);

   tam = strlen(banco);
   if (tam != 3) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->banco, banco, 3);

   tam = strlen(banco_dest);
   if (tam != 3) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->banco_dest, banco_dest, 3);

   tam = strlen(agencia_dest);
   if (tam != 4) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->agencia_dest, agencia_dest, 4);

   tam = strlen(conta_dest);
   if (tam != 10) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   memcpy(ptr->conta_dest, conta_dest, 10);

   return (chama_servidor(&m));
}

#ifdef ANSI
int esp_tecla_pchek(char *seg, char *tecla)
#else
int esp_tecla_pchek(seg, tecla)
   char *seg;
   char *tecla;
#endif
{
   MESSAGE m;
   struct p_cmd23 *ptr;
   int tam;

   ptr = (struct p_cmd23 *)&m.dados[0];
   ptr->cmd = P_ESP_TECLA_PCHEK;

   tam = strlen(seg);
   if ((tam != 3) ||
       ((seg[0] - 0x30) * 100 + (seg[1] - 0x30) * 10 + (seg[2] - 0x30) > 255))
   {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   memcpy(ptr->seg, seg, 3);

   if (chama_servidor(&m) == FALSE)
      return (FALSE);

   if (m.estado == P_TERMINO_OK)
   {
      memcpy(tecla, &m.dados[4], m.tam - 4);
      tecla[m.tam - 4] = 0;
      return (TRUE);
   }
   else
      return (FALSE);
}


#ifdef ANSI
int envia_mostra_com_lcd(char tipo_teclado, char *poscur, char cursor,
                         char *timeout, char dir_desloc, char *mensagem,
                         char *resp)
#else
int envia_mostra_com_lcd(tipo_teclado, poscur, cursor, timeout, dir_desloc,
                         mensagem, resp)
     char tipo_teclado;
     char *poscur;
     char cursor;
     char *timeout;
     char dir_desloc;
     char *mensagem;
     char *resp;
#endif
{
   int tam;
   struct p_cmd29 *ptr;
   MESSAGE m;

   ptr = (struct p_cmd29 *)&m.dados[0];
   ptr->cmd = P_ESPERA_MOSTRA_TECLA;

   tam = strlen(poscur);
   if (tam != 2) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   tam = strlen(timeout);
   if (tam != 3) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }
   
   tam = strlen(mensagem);
   if (tam > 16) {
      erro_perto = P_ERRO_PARAMETRO;
      return (FALSE);
   }

   ptr->tipo_teclado = tipo_teclado;
   memcpy(ptr->poscur, poscur, 2);
   ptr->cursor = cursor;
   memcpy(ptr->timeout, timeout, 3);
   ptr->dir_desloc = dir_desloc;
   memset(ptr->mensagem, ' ', 16);
   memcpy(ptr->mensagem, mensagem, tam);
   tam = tam + 9;
   m.tam = tam;

   if (chama_servidor(&m) == FALSE)
	{
      /*erro_perto = P_ERRO_CMC;*/
      return (FALSE);
	}

   if (m.estado == P_TERMINO_OK)
   {
      memcpy(resp, &m.dados[4], m.tam-4);
      resp[m.tam-4] = 0;
      return (TRUE);
   }
}


#ifdef ANSI
int alinha_e_le_cod_barras(char *cod_barras)
#else
int alinha_e_le_cod_barras(cod_barras)
char *cod_barras;
#endif
{
   int nTam;
   struct p_cmd0 *ptr;
   MESSAGE m;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ALINHA_LE_DOC_BARRAS;

   if (chama_servidor(&m) == FALSE) {
      return (FALSE);
	}

   if (m.estado == P_TERMINO_OK)
   {
      nTam = strlen(m.dados);
      memcpy(cod_barras, m.dados, nTam);
      cod_barras[nTam] = 0;
      return (TRUE);
   }

   return (FALSE);
}


/*
 * Retorno da funo envia_status_equip:
 * '0' - se  a primeira ocorrncia deste comando aps o equipamento ter sido 
 *       ligado
 * '1' - se este comando j foi enviado desde que o equipamento foi ligado pela 
 *       ltima vez
*/
#ifdef ANSI
int envia_status_equip(char *status)
#else
int envia_status_equip(status)
   char *status;
#endif
{
   struct p_cmd0 *ptr;
   MESSAGE m;

   ptr = (struct p_cmd0 *)&m.dados[0];
   ptr->cmd = P_ENVIA_STATUS_EQUIP;

   if (chama_servidor(&m) == FALSE) {
      return (FALSE);
	}

   if (m.estado == P_TERMINO_OK)
   {
      status[0] = m.dados[1];
      status[1] = 0;

      return (TRUE);
   }

   return (FALSE);
}

#ifdef ANSI
int estado_perto(char *msg)
#else
int estado_perto(msg)
     char *msg;
#endif
{
   int i;
   static struct
   {
      int cod_erro;
      char *msg_erro;
   } msg_erro[] = {
      { P_TERMINO_OK, "Sucesso na execucao do comando"} ,
      { P_ERRO_DADOS, "Mensagem com dados invalidos"} ,
      { P_ERRO_TAMANHO, "Tamanho da mensagem invalido"} ,
      { P_ERRO_CMC_INV, "Leitura dos caracteres magneticos invalida"} ,
      { P_ERRO_MOTOR1, "Problemas no acionamento do Motor 1"} ,
      { P_ERRO_JA_MARCADO, "Documento ja Pos-Marcado"} ,
      { P_ERRO_MOTOR2, "Problemas no acionamento do Motor 2"} ,
      { P_ERRO_BANCO, "Banco diferente do solicitado"} ,
      { P_ERRO_FITA, "Fita Magnetica no fim ou ausente"} ,
      { P_ERRO_SENSOR1, "Sensor 1 obstruido"} ,
      { P_ERRO_SENSOR2, "Sensor 2 obstruido"} ,
      { P_ERRO_SENSOR3, "Sensor 4 obstruido"} ,
      { P_ERRO_CABECA, "Erro no posicionamento da cabeca de escrita"} ,
      { P_ERRO_DIG_VER, "Digito verificador do cheque nao confere"} ,
      { P_ERRO_SEM_CAR_MAG, "Ausencia de caracteres magneticos no cheque"} ,
      { P_ERRO_SEM_DOC, "Documento nao inserido na maquina"} ,
      { P_ERRO_INSERCAO, "Documento mal inserido"} ,
      { P_ERRO_ALINHA, "Cheque preso durante o alinhamento"} ,
      { P_ERRO_ALINHA_S1, "Cheque preso durante o alinhamento: "
			"S1 obstruido e S2 desobstruido"}
      , { P_ERRO_ALINHA_S2, "Cheque preso durante o alinhamento: "
			"S1 desobstruido e S2 obstruido"} ,
      { P_ERRO_ALINHA_S12, "Cheque preso durante o alinhamento: "
			"S1 e S2 obstruidos"} ,
      { P_ERRO_PREENCH, "Cheque preso durante o preenchimento"} ,
      { P_ERRO_PREENCH_S1, "Cheque preso durante o preenchimento: "
			"S1 obstruido e S2 desobstruido"},
      { P_ERRO_PREENCH_S2, "Cheque preso durante o preenchimento: "
			"S1 desobstruido e S2 obstruido"} ,
      { P_ERRO_PREENCH_S12, "Cheque preso durante o preenchimento: "
			"S1 e S2 obstruidos"} ,
      { P_ERRO_CARAC_INEXIST, "Caractere inexistente"} ,
      { P_ERRO_LISTA_CHEIA, "Lista Negra interna cheia"} ,
      { P_ERRO_RELOGIO, "Relogio ausente ou nao funcionando"} ,
      { P_ERRO_CHQ_AUSENTE, "Cheque ausente"} ,
      { P_ERRO_PIN_TECL_AUSENTE, "Pin Pad ou teclado ausente"} ,
      { P_ERRO_TRANSMISSAO, "Erro de transmissao"} ,
      { P_ERRO_TX_IMPRESSORA, "Erro de transmissao: impressora off line, "
			"desconectada ou ocupada"} ,
      { P_ERRO_PINPAD, "Erro no Pin Pad"} ,
      { P_ERRO_CHQ_NA_LISTA, "Cheque na Lista Negra"} ,
      { P_ERRO_CHQ_FORA_LISTA, "Cheque nao encontrado na Lista Negra"} ,
      { P_ERRO_CANCELADO, "Comando cancelado"} ,
      { P_ERRO_CAD_CHEIO, "Tabela de cadastro de cheques cheia"} ,
      { P_ERRO_CAD, "Cheque nao se encontra no cadastro de "
         "layouts de cheques"} ,
      { P_ERRO_LEITURA_CARTAO, "Leitura de cartao invalida"} ,
      { P_ERRO_LEITURA_TRILHA1, "Erro na Leitura da trilha1"} ,
      { P_ERRO_LEITURA_TRILHA2, "Erro na Leitura da trilha2"} ,
      { P_ERRO_LEITURA_TRILHA3, "Erro na Leitura da trilha3"} ,
      { P_ERRO_CMC, "Leitura magnetica incorreta (cheque virado)"} ,
      { P_ERRO_EOT, "PINPAD nao retornou EOT"} ,
      { P_ERRO_NAK, "PINPAD retornou NAK"} ,
      { P_ERRO_SEM_RESPOSTA, "PINPAD nao responde"} ,
      { P_ERRO_TEMPO_ESGOTADO, "Tempo esgotado"} ,
      { P_ERRO_CMD_INEXISTENTE, "Comando inexistente"} ,
      { P_ERRO_RX, "Erro de comunicacao com PertoCheck (RX)"} ,
      { P_ERRO_TX, "Erro de comunicacao com PertoCheck (TX)"} ,
      { P_ERRO_INVALIDO, "PertoCheck informou codigo de erro invalido"} ,
      { P_ERRO_MODELO, "Comando nao suportado neste modelo de PertoCheck"} ,
      { P_ERRO_SEND, "Erro de comunicacao com servidor (SEND)"} ,
      { P_ERRO_RECEIVE, "Erro de comunicacao com servidor (RECEIVE)"} ,
      { P_ERRO_PARAMETRO, "Erro nos parametros passados"} ,
      { P_ERRO_NOME_PERTO, "Nao existe Pertochek com o nome solicitado"} ,
		{ P_ERRO_INIT_SERIAL, "Falha inicializacao conexao serial"} ,
		{ P_ERRO_CONFIG, "Falha no arquivo de configuracao"} ,
      { -1, NULL}
	};

   i = 0;

   while (msg_erro[i].cod_erro != -1) {
      if (msg_erro[i].cod_erro == erro_perto) {
         if (msg != NULL)
            strcpy(msg, msg_erro[i].msg_erro);
         return (msg_erro[i].cod_erro);
      }
      i++;
   }

#ifndef PERTO_NOT_STANDARD
   if (msg != NULL)
      strcpy(msg, "Codigo de erro invalido");
#endif /* PERTO_NOT_STANDARD */
   return (-1);
}

/*#######################  C O M U N I C A C A O  ############################*/

#ifdef ANSI
static int chama_servidor(MESSAGE * ptr)
#else
static int chama_servidor(ptr)
     MESSAGE *ptr;
#endif
{
#ifndef PERTO_NO_SERVER_DAEMON
	int serverqid;
	int clientqid;

   if ((serverqid = msgget(chave_servidor, 0666)) == -1) {
     	erro_perto = P_ERRO_SEND;
     	ptr->estado = P_ERRO_SEND;
     	return (FALSE);
	}

   if ((clientqid = open_queue(chave_cliente, 0)) == -1) {
     	erro_perto = P_ERRO_SEND;
     	ptr->estado = P_ERRO_SEND;
      return (FALSE);
   }

#ifndef PERTO_OLDCLT_COMPAT
   ptr->chave_cliente = clientqid;
#else /* PERTO_OLDCLT_COMPAT */
   ptr->chave_cliente = chave_cliente;
#endif /* PERTO_OLDCLT_COMPAT */

   if (send(serverqid, ptr, sizeof(MESSAGE)) == FALSE) {
      erro_perto = P_ERRO_SEND;
      ptr->estado = P_ERRO_SEND;
      return (FALSE);
   }

   if (receive(clientqid, ptr, sizeof(MESSAGE)) == FALSE) {
      erro_perto = P_ERRO_RECEIVE;
      ptr->estado = P_ERRO_RECEIVE;
      return (FALSE);
   }
#else /* PERTO_NO_SERVER_DAEMON */
   talk_to_pertochek(ptr);
#endif /* PERTO_NO_SERVER_DAEMON */

   erro_perto = ptr->estado;

   if ((ptr->estado != P_TERMINO_OK) &&
       (ptr->estado != P_ERRO_CHQ_NA_LISTA) &&
       (ptr->estado != P_ERRO_CHQ_FORA_LISTA) &&
       (ptr->estado != P_ERRO_LEITURA_TRILHA1) &&
       (ptr->estado != P_ERRO_LEITURA_TRILHA2) &&
       (ptr->estado != P_ERRO_LEITURA_TRILHA3))
      return (FALSE);
   else
      return (TRUE);
}

#ifndef PERTO_NO_SERVER_DAEMON

#ifdef ANSI
static int open_queue(long key, int fordel)
#else
static int open_queue(key, fordel)
     long key; int fordel;
#endif
{
   static struct
   {
      long key;
      int qid;
   } queues[MAXOPEN];
   int i;
   int avail;
   int qid;

   avail = -1;

   for (i = 0; i < MAXOPEN; i++)
   {
      if (queues[i].key == key)
		{
			if (fordel)
      		queues[i].key = 0;
         return (queues[i].qid);
		}
      if (queues[i].key == 0 && avail == -1)
      {
         avail = i;
         break;
      }
   }

   if (avail == -1)
   {
      errno = 0;
      return (-1);
   }

#ifndef PERTO_OLDCLT_COMPAT
   if ((qid = msgget(0L, 0666 | IPC_CREAT | IPC_PRIVATE)) == -1)
#else /* PERTO_OLDCLT_COMPAT */
   if ((qid = msgget(key, 0666 | IPC_CREAT)) == -1)
#endif /* PERTO_OLDCLT_COMPAT */
      return (-1);

   queues[avail].key = key;
   queues[avail].qid = qid;

   return (qid);
}

#ifdef ANSI
static int send(int qid, MESSAGE * p_buf, int nbytes)
#else
static int send(qid, p_buf, nbytes)
   int qid;
   MESSAGE *p_buf;
   int nbytes;
#endif
{
   int ret;

	/*
   if ((qid = open_queue(dest_key)) == -1)
      return (FALSE);
	*/

   p_buf->tipo_msg = 1L;

   ret =
      msgsnd(qid, (struct msgbuf *)p_buf, nbytes - sizeof(p_buf->tipo_msg), 0);

   if (ret == -1)
      return (FALSE);
   else
      return (TRUE);
}

#ifdef ANSI
static int receive(int qid, MESSAGE * p_buf, int nbytes)
#else
static int receive(qid, p_buf, nbytes)
   int qid;
   MESSAGE *p_buf;
   int nbytes;
#endif
{
   int ret;

   p_buf->tipo_msg = 1L;

   ret =
      msgrcv(qid, (struct msgbuf *)p_buf, nbytes - sizeof(p_buf->tipo_msg), 0L,
             0);

   if (ret == -1)
      return (FALSE);
   else
      return (TRUE);
}

#ifdef ANSI
static int remove_queue(long key)
#else
static int remove_queue(key)
   long key;
#endif
{
   int qid;

   if ((qid = open_queue(key, 1)) == -1)
      return (FALSE);

   if (msgctl(qid, IPC_RMID, NULL) == -1)
      return (FALSE);
   else
      return (TRUE);
}

#endif /* PERTO_NO_SERVER_DAEMON */

/*#######################  M I S C E L A N E A  ##############################*/

#ifdef ANSI
static void pnum(char *s, int i, int w, int base)
#else
int pnum(s, i, w, base)
   char *s;
   int i;
   int w;
   int base;
#endif
{
   int f;
   char str[NDIG];
   char *cp;

   cp = &str[NDIG];

   while (i)
   {
      *--cp = "0123456789ABCDEF"[i % base];
      i /= base;
   }

   f = (&str[NDIG] - cp);

   while (w-- > f)
      *s++ = '0';

   while (f--)
      *s++ = *cp++;
}

#ifdef ANSI
static void anum(int *sai, char *val, int tam)
#else
int anum(sai, val, tam)
   int *sai;
   char *val;
   int tam;
#endif
{
   char temp[16];

   memcpy(temp, val, tam);
   temp[tam] = 0;
   *sai = atoi(temp);
}

/* cliente.c eof --------------------------------------------------------- */
