/* -------------------------------------------------------------------------
 * Perto S/A
 * File:    tpchkpin.c
 * Purpose: Testing the access to a Pertochek interleaved with access to
 *          a Verifone SC552 PINPAD.
 *          Access to Pertochek should be performed with #dtroff set it the
 *          configuration file. PINPAD access will be performed with DTR on,
 *          so that Pertochek will be bypassed and we will be able to talk
 *          directly to the PINPAD...
 * Programmer: Tarcisio Kreutz
 * Date:    2003/04/15.
 * Changes:
 * ------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <perto.h>

#include "testpind.h"

#ifndef PERTO_NOT_STANDARD
static char arq_conf[] = { "arq_conf.dat" };
#else
#ifdef DEBUG
static char arq_conf[] = { "./arq_conf.dat" };
#else
static char arq_conf[] = { "/u/cartlink/arq_conf.dat" };
#endif
#endif /* PERTO_NOT_STANDARD */

char *currency = "REAIS";

/* */
#ifdef ANSI
static void showerror(char *caller)
#else
static void showerror(caller)
	char *caller;
#endif /* ANSI */
{
	int code;
	char message[80];

	code = estado_perto(message);

	fprintf(stderr, "Falha em [%s]: [%d - %s].\n", caller, code, message);
}	/* showerror() */

#ifdef ANSI
static void testpertochek_check(int msglevel)
#else
static void testpertochek_check(msglevel)
	int msglevel;
#endif /* ANSI */
{
	char buffer256[256];
	time_t timecount;
	struct tm *ltm;
	/* ------------------- */

	if (msglevel > 1)
		fprintf(stdout, "Obtendo versao firmware Pertochek...\n");
	/* (Shouldn't the name of this function be "obtem_versao()"?) */
	if (! envia_versao(buffer256))
	{
		showerror("envia_versao()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Versao firmware Pertochek: %s\n", buffer256);

	if (msglevel > 1)
		fprintf(stdout, "Programando telefone...\n");
	if (! prg_telefone("0051", "0004898700"))
	{
		showerror("prg_telefone()");
		return;
	}

	if (msglevel > 1)
		fprintf(stdout, "Programando cidade...\n");
	if (! prg_cidade("PORTO ALEGRE"))
	{
		showerror("prg_cidade()");
		return;
	}

	if (msglevel > 1)
		fprintf(stdout, "Verificando cidade...\n");
	/* (Shouldn't the name of this function be "obtem_cidade()"?) */
	if (! envia_cidade(buffer256))
	{
		showerror("envia_cidade()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Cidade programada: %s\n", buffer256);

	if (msglevel > 1)
		fprintf(stdout, "Programando data do cheque...\n");
	timecount = time(NULL);
	ltm = localtime(&timecount);
	sprintf(buffer256, "%02d%02d%02d", 
			ltm->tm_mday, ltm->tm_mon + 1, ltm->tm_year % 100);
	if (! prg_data_chk(buffer256))
	{
		showerror("prg_data_chk()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Data programada: %s\n", buffer256);

	if (msglevel > 1)
		fprintf(stdout, "Programando beneficiario...\n");
	if (! prg_benef("FULANO OU SICLANO E TAL"))
	{
		showerror("prg_benef()");
		return;
	}

	if (msglevel > 1)
		fprintf(stdout, "Verificando beneficiario...\n");
	/* (Shouldn't the name of this function be "obtem_beneficiario()"?) */
	if (! envia_benef(buffer256))
	{
		showerror("envia_benef()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Beneficiario: %s\n", buffer256);

	if (msglevel > 1)
		fprintf(stdout, "Programando cruzamento do cheque...\n");
	if (! prg_colunas("CRUZAMENTO 1", "CRUZAMENTO 2", "CRUZAMENTO 3"))
	{
		showerror("prg_colunas()");
		return;
	}

	if (msglevel > 1)
		fprintf(stdout, "Programando moeda...\n");
	if (! prg_moeda(currency))
	{
		showerror("prg_moeda()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Moeda: %s\n", currency);

	if (msglevel > 1)
		fprintf(stdout, "Programando \"BOM PARA\"\n");
	sprintf(buffer256, "BOM PARA: %02d/%02d/%02d",
			ltm->tm_mday < 28 ? ltm->tm_mday : 28,
			ltm->tm_mon < 11 ? ltm->tm_mon + 1 : 1,
			(ltm->tm_mon < 11 ? ltm->tm_year : ltm->tm_year + 1) % 100);
	if (! prg_bom_para(buffer256))
	{
		showerror("prg_bom_para()");
		return;
	}

	if (msglevel)
		fprintf(stdout, "%s\n", buffer256);
}	/* testpertochek_check() */

#ifdef ANSI
static void testpertochek_pin(int msglevel)
#else
static void testpertochek_pin(msglevel)
	int msglevel;
#endif /* ANSI */
{
	char buffer256[256];
	char buffer64[64];
	time_t timecount;
	struct tm *ltm;
	/* ------------------- */

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Q2 ao PINPAD, via Pertochek...\n");
	if (! envia_com_pinpad("060", '0', '0', '1', '0', "Q2", buffer256))
	{
		showerror("envia_com_pinpad(Q2)");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Resposta ao Q2: [%s]\n", buffer256);

	sleep(2);

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Z8 ao PINPAD, via Pertochek...\n");
	if (! envia_com_pinpad("060", '0', '0', '1', '0', 
				"Z8TESTANDO PIN VIA PERTOCHEK", buffer256))
	{
		showerror("envia_com_pinpad(Z8)");
		return;
	}

	if (msglevel)
		fprintf(stdout, "Resposta ao Z8: [%s]\n", buffer256);

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Z2 ao PINPAD, via Pertochek...\n");
	timecount = time(NULL);
	ltm = localtime(&timecount);
	sprintf(buffer64, "Z2\x1a%02d%02d%04d  %02d%02d%02d",
			ltm->tm_mday, ltm->tm_mon + 1, ltm->tm_year + 1900,
			ltm->tm_hour, ltm->tm_min, ltm->tm_sec);
	if (! envia_com_pinpad("060", '0', '0', '1', '0', buffer64, buffer256))
	{
		showerror("envia_com_pinpad(Z2)");
		return;
	}
} 	/* testpertochek_pin() */

#ifdef ANSI
static void testdirect_pin(int handle, int msglevel)
#else
static void testdirect_pin(handle, msglevel)
	int handle; int msglevel;
#endif /* ANSI */
{
	char buffer256[256];
	time_t timecount;
	struct tm *ltm;
	/* ------------------- */

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Q2 ao PINPAD, diretamente...\n");
	strcpy(buffer256, "Q2");
	if (pin_send(handle, buffer256, strlen(buffer256), 
			PINPAD_PROTO_STXETX, PINPAD_EOT_NO) != 0)
	{
		return;
	}

	sleep(3);

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Z8 ao PINPAD, diretamente...\n");
	strcpy(buffer256, "Z8TESTANDO PIN DIRETAMENTE");
	if (pin_send(handle, buffer256, strlen(buffer256), 
			PINPAD_PROTO_STXETX, PINPAD_EOT_NO) != 0)
	{
		return;
	}

	if (msglevel > 1)
		fprintf(stdout, "Enviando comando Z2 ao PINPAD, diretamente...\n");
	timecount = time(NULL);
	ltm = localtime(&timecount);
	sprintf(buffer256, "Z2\x1a%02d%02d%04d--%02d%02d%02d",
			ltm->tm_mday, ltm->tm_mon + 1, ltm->tm_year + 1900,
			ltm->tm_hour, ltm->tm_min, ltm->tm_sec);
	if (pin_send(handle, buffer256, strlen(buffer256), 
			PINPAD_PROTO_STXETX, PINPAD_EOT_NO) != 0)
	{
		return;
	}
}	/* testdirect_pin() */

/* */
#ifdef ANSI
int main(int argc, char *argv[])
#else
int main(argc, argv)
	int argc; char *argv[];
#endif /* ANSI */
{
	char *machine;
	int nexec;
	int msglevel;
	int pinhandle;
	int errcode;
	st_serverinfo_t srvcfg;
	/* ----------------- */

	if (argc < 2)
	{
		fprintf(stderr, 
			"Execute: \"%s <Pertochek> <NR> <V>\"\n"
			"\tPertochek: Nome da Pertochek no arquivo de configuracao\n"
			"\tNR: Numero de repeticoes do teste (se 0: sem limite)\n"
			"\tV:  Nivel de mensagens informativas sobre execucao (0, 1 ou 2)\n"
			"\t\t0: Apenas erros\n"
			"\t\t1: Dados obtidos como resultados de operacoes\n"
			"\t\t2: Chamadas de operacoes e seus resultados\n",
				argv[0]);
		return -1;
	}

	machine = argv[1];

	if (argc > 2)
		nexec = atoi(argv[2]);
	else
		nexec = 1;

	if (argc > 3)
		msglevel = atoi(argv[3]);
	else
		msglevel = 2;

	if (msglevel > 1)
		fprintf(stdout, "Obtendo configuracao serial Pertochek->PINPAD...\n");
	errcode = 0;
   if (! pcheck_get_config(machine, arq_conf, &srvcfg, &errcode))
   {
		fprintf(stderr, "Falha obtencao de configuracao [%s: %d].\n",
				arq_conf, errcode);

		return -1;
   }

	if (! (srvcfg.pchekcaps & PCHEK_CAPS_DTROFF))
	{
		fprintf(stderr, 
				"ATENCAO: O acesso direto ao PINPAD nao vai funcionar porque o\n"
				"acesso a Pertochek [%s] nao esta configurado com DTR OFF,\n"
				"no arquivo de configuracao [%s].\n", machine, arq_conf);
	}

	/* ---------- */
	do
	{
		if (msglevel > 1)
			fprintf(stdout, "Inicializando comunicacao com Pertochek...\n");
		if (! inic_cliente(machine))
		{
			showerror("inic_cliente()");
			continue;
		}

		testpertochek_check(msglevel);
		testpertochek_pin(msglevel);

		if (msglevel > 1)
			fprintf(stdout, "Encerrando comunicacao com Pertochek...\n");
		if (! fim_cliente())
		{
			showerror("fim_cliente()"); /* Error code will not be consistent */
			continue;
		}
		
		if (msglevel > 1)
			fprintf(stdout, "Iniciando comunicacao com PINPAD (1200 bps)...\n");
		if ((pinhandle = pin_open_dev(srvcfg.device, 1200)) < 0)
		{
			continue;
		}

		testdirect_pin(pinhandle, msglevel);

		if (msglevel > 1)
			fprintf(stdout, "Encerrando comunicacao direta com PINPAD...\n");
		pin_close_dev(pinhandle);
	} while ((! nexec) || (--nexec > 0));

	return 0;
}	/* main() */

/* tpchkpin.c -------------------------------------------------------------- */
