/* 28/10/96 - 15:42
 ******************************************************************************
 **
 **  MODULO : ipc.c
 **
 **  FUNCAO : tratamento da comunicacao inter-processos
 **
 ******************************************************************************
 *
 ******************************************************************************
 **
 **  Registro de Alteracoes/Versoes :
 **  ==============================
 **
 **  VERSAO   DATA    RESPONSAVEL   DESCRICAO
 **
 **  V1.00  11/06/96  Magoga        Criacao do arquivo
 **  V1.01  2003/01/24 Tarcisio Kreutz, Perto
 **     - Better error message detail. 
 **       (Error info should receive additional improvements!).
 **     - Previous version stored client msg queue ids in static area.
 **       Than, if the same client (same pid == same key) allocated a new
 **       msg queue to replace a previous one, the server was not able to
 **       write the response to the client's queue.
 **       Fixed in two modes: 
 **         - PERTO_OLDCLT_COMPAT defined: The server gets the queue id each 
 **       time it sends a response. Client continues to use its PID as the 
 **       msg key.
 **         - PERTO_OLDCLT_COMPAT not defined: The client allocates its
 **       queue with IPC_PRIVATE and sends the msg queue ID to the server,
 **       instead of the msg queue KEY.
 **     - TODO: IPC receive() performance could be improved (no open_queue() 
 **       would be needed if the server kept/used its msg queue ID instead of 
 **       msg queue KEY). No time for this change now...
 **  V2.06  2005/04/18 MEGS, Perto
 **     - No emite mensagens no "stderr" quando ocorre erros.
 **
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  COMENTARIOS  :                                                           *
 **                                                                           *
 **                                                                           *
 **  ATENCAO :                                                                *
 **  =======                                                                  *
 **                                                                           *
 **                                                                           *
 ******************************************************************************
 */

/*#######################  I N C L U D E S  ##################################*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>
extern int errno;

#include "perto.h"

/*#######################  C O N S T A N T E S  ##############################*/

#define MAXOPEN     20

/*#######################  P R O T O T I P O S  ##############################*/

#ifdef ANSI
static int open_queue(long);
#endif

/*#######################  V A R I A V E I S  ################################*/

/*=======================  L o c a i s  ======================================*/

/*=======================  P u b l i c a s  ==================================*/

/*=======================  E x t e r n a s  ==================================*/

extern int errno;

/*#######################  R O T I N A S  #####################################*/
#ifdef ANSI
int open_queue(long key)
#else
int open_queue(key)
     long key;
#endif
{
   static int kneedinit = 1;
   static struct
   {
      long key;
      int qid;
   }
   queues[MAXOPEN];
   int i;
   int avail;
   int qid;

   if (kneedinit)
   {
      memset((char *)queues, 0, sizeof(queues));
      kneedinit = 0;
   }

   avail = -1;

   for (i = 0; i < MAXOPEN; i++)
   {
      if (queues[i].key == key)
         return (queues[i].qid);
      if (queues[i].key == 0 && avail == -1)
      {
         avail = i;
         break;
      }
   }

   if (avail == -1)
   {
      errno = 0;
      fprintf(stderr,
              "ERRO: Falta espaco para fila de mensagens adicional. "
              "[Maximo: %d]\n", MAXOPEN);
      return (-1);
   }

   if ((qid = msgget(key, 0666 | IPC_CREAT)) == -1)
   {
      fprintf(stderr, "ERRO: Falha acesso fila de mensagens [%#lx: %s]\n",
              key, strerror(errno));
      return (-1);
   }

   queues[avail].key = key;
   queues[avail].qid = qid;

   return (qid);
}

#ifdef ANSI
int send(long dest_key, MESSAGE * p_buf, int nbytes)
#else
int send(dest_key, p_buf, nbytes)
     long dest_key;
     MESSAGE *p_buf;
     int nbytes;
#endif
{
   int qid;
   int ret;

#ifndef PERTO_OLDCLT_COMPAT
	qid = (int)dest_key;
#else /* PERTO_OLDCLT_COMPAT */
	/* THIS DOES NOT WORK IF CLIENT KEEPS QUEUE KEY, BUT CHANGES QUEUE ID. 
   if ((qid = open_queue(dest_key)) == -1)
      return (FALSE);
	*/

   if ((qid = msgget(dest_key, 0666)) == -1) {
      dest_key, qid, strerror(errno));
      return (-1);
   }
#endif /* PERTO_OLDCLT_COMPAT */

   p_buf->tipo_msg = 1L;

   ret =
      msgsnd(qid, (struct msgbuf *)p_buf, nbytes - sizeof(p_buf->tipo_msg), 0);

   if (ret == -1)
      return (FALSE);
   else
      return (TRUE);
}

#ifdef ANSI
int receive(long src_key, MESSAGE * p_buf, int nbytes)
#else
int receive(src_key, p_buf, nbytes)
     long src_key;
     MESSAGE *p_buf;
     int nbytes;
#endif
{
   int qid;
   int ret;

   if ((qid = open_queue(src_key)) == -1)
      return (FALSE);

   p_buf->tipo_msg = 1L;


   ret =
      msgrcv(qid, (struct msgbuf *)p_buf, nbytes - sizeof(p_buf->tipo_msg), 0L,
             0);

   if (ret == -1)
      return (FALSE);
   else
      return (TRUE);
}

#ifdef ANSI
int remove_queue(long key)
#else
int remove_queue(key)
     long key;
#endif
{
   int qid;

   if ((qid = open_queue(key) == -1) || msgctl(qid, IPC_RMID, NULL) == -1)
      return (FALSE);
   else
      return (TRUE);
}

/* ipc.c eof ---------------------------------------------------------------- */
