/* 19/12/96 10:32
 ******************************************************************************
 **
 **  MODULO : server.c
 **
 **  FUNCAO : programa principal do servidor
 **
 ******************************************************************************
 *
 ******************************************************************************
 **
 **  Registro de Alteracoes/Versoes :
 **  ==============================
 **
 **  VERSAO   DATA    RESPONSAVEL   DESCRICAO
 **
 **  V1.00  11/06/96  Magoga        Criacao do arquivo
 **  V2.00  03/04/98  Borba         Inclusao novos comandos p/LCD, PINPAD,
 **                                 cartao magnetico e impressora.
 **  V2.01  2003/04/11, Tarcisio Kreutz, Perto S.A.
 **      - Inserted "Borba new commands" into Perto sources.
 **      - Conditional compilation:
 **  				+ PERTO_NOT_STANDARD: "Translates" the '=' command to the 'P' 
 **              command [alinha_e_le()] as changed by Qualita.
 **            + PERTO_NO_SERVER_DAEMON: The "server" code is linked to the
 **              "client" code, so that no server daemon process is needed.
 **      - If 'dtroff' mode is set, open/close the serial device for each 
 **        command.  This is needed because there may exist PINPAD access 
 **        software that wants to access the same serial port (since the new 
 **        Pertochek has a mode (DTR off) that lets serial data to reach its 
 **        PIN port directly).
 **      - If 'dtroff' mode is set, "talk" to Pertochek with serial DTR off.
 **      - The 'dtroff' mode may be set in the configuration file. See conf.c.
 **
 **  V2.05  ????/??/??, Flvio Oliveira, Perto S.A.
 **      - P_IMP_COPIA: 
 **      - Removidas as seguintes funes: P_POS_MARC_BENEF, P_POR_MARC, 
 **        P_POS_MARC_BENEF_PAR, P_POS_MAR_PAR, P_ENVIA_DATA, P_ENVIA_HORA,
 **        P_PRG_NUM_MAQ, P_PRG_DATA e P_PRG_HORA.
 **      - Incluidas as seguintes funes: P_ENVIA_ULTIMO_CMC7, P_LISTA_NEGRA,
 **        P_IMP_FOLHA_PARCIAL, P_IMP_COMPLETO_CHQ_TB, P_IMP_PARCIAL_CHQ_TB,
 **        P_ESP_TECLA_PCHEK, P_ESPERA_MOSTRA_TECLA, P_ALINHA_LE_DOC_BARRAS e
 **        P_ENVIA_STATUS_EQUIPE.
 **
 **  V2.06  2005/03/28, Miguel Silva, Perto S.A.
 **      - Diminuido timeout de comandos rpidos.
 **      - P_IMP_FOLHA e P_IMP_FOLHA_PARCIAL foram alterados. O caracter 0xFF 
 **      agora  inserido pelo client.
 **      - P_LISTA_NEGRA foi corrigido.
 **      - Nomenclatura de alguns erros foram alteradas para ficarem mais 
 **      legveis.
 **
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  COMENTARIOS  :                                                           *
 **                                                                           *
 **                                                                           *
 **  ATENCAO :                                                                *
 **  =======                                                                  *
 **                                                                           *
 **                                                                           *
 ******************************************************************************
 */

/*#######################  I N C L U D E S  ##################################*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "perto.h"

/*#######################  C O N S T A N T E S  ##############################*/

/* #define DEBUG		1 */

#define TMOUT_TX	10            /* 10 seg. */
#define TMOUT_COMANDO1  1       /*  1 seg. */
#define TMOUT_COMANDO2  90      /* 1,5 min. */

/*#######################  P R O T O T I P O S  ##############################*/

#ifdef ANSI
# ifndef PERTO_NO_SERVER_DAEMON
static void talk_to_pertochek(st_serverinfo_t *);
# endif /* PERTO_NO_SERVER_DAEMON */
static void comando1(MESSAGE *, int, int);
static void comando2(MESSAGE *, int, int);
static void comando3(MESSAGE *, int, int);
static void comando4(MESSAGE *, int, int);
static void comando5(MESSAGE *, int, int);
static void comando6(MESSAGE *, int, int);
static int trata_erro(char *);
static int acerta_tam_moeda(MESSAGE *);
# ifdef DEBUG
void dump(char *, char *, int);
# else
#  define dump(a, b, c)
# endif /* DEBUG */
#else
# ifndef PERTO_NO_SERVER_DAEMON
static void talk_to_pertochek();
# endif /* PERTO_NO_SERVER_DAEMON */
static void comando1();
static void comando2();
static void comando3();
static void comando4();
static void comando5();
static void comando6();
static int trata_erro();
static int acerta_tam_moeda();
#ifdef DEBUG
int dump();
#else
#define dump(a, b, c)
#endif /* DEBUG */
#endif /* ANSI */

#ifdef ANSI
extern int rec_stxetx(char *, int *, int);
extern int trans_stxetx(char *, int, int);
extern int inic_stxetx(char *, int, int);
extern int stop_stxetx(void);
extern int inic_conf(st_config_t *, char *);
#else
extern int rec_stxetx();
extern int trans_stxetx();
extern int inic_stxetx();
extern int stop_stxetx();
extern int inic_conf();
#endif

/*#######################  V A R I A V E I S  ################################*/

/*=======================  L o c a i s  ======================================*/

static st_config_t conf;
#ifndef PERTO_NOT_STANDARD
static char arq_conf[] = { "arq_conf.dat" };
#else
#ifdef DEBUG
static char arq_conf[] = { "./arq_conf.dat" };
#else
static char arq_conf[] = { "/u/cartlink/arq_conf.dat" };
#endif
#endif /* PERTO_NOT_STANDARD */

/*=======================  P u b l i c a s  ==================================*/

/*=======================  E x t e r n a s  ==================================*/

/*#######################  R O T I N A S  ####################################*/
#ifndef PERTO_NO_SERVER_DAEMON

#ifdef ANSI
int main(void)
#else
int main()
#endif
{
   int i;
   int pid;
   int pids[MAX_PERTO];

   if (inic_conf(&conf, arq_conf) == FALSE) {
      fprintf(stderr, "ERRO: arquivo de configuracao.\n");
      exit(1);
   }

#ifdef DEBUG
   talk_to_pertochek(&conf.info[0]);
#else
   for (i = 0; i < conf.num_imp; i++) {
      if ((pids[i] = fork()) == 0)
         talk_to_pertochek(&conf.info[i]);
   }

   while (1) {
      pid = wait(NULL);

      for (i = 0; i < conf.num_imp; i++) {
         if (pids[i] == pid)
            break;
      }

      if (i >= conf.num_imp)
         continue;

      if ((pids[i] = fork()) == 0)
         talk_to_pertochek(&conf.info[i]);
   }
#endif
}

#endif /* PERTO_NO_SERVER_DAEMON */

#ifndef PERTO_NO_SERVER_DAEMON
#ifdef ANSI
static void talk_to_pertochek(st_serverinfo_t * srvcfg)
#else
static void talk_to_pertochek(srvcfg)
	st_serverinfo_t * srvcfg;
#endif /* ANSI */
#else /* PERTO_NO_SERVER_DAEMON */
#ifdef ANSI
void talk_to_pertochek(MESSAGE * pmessage)
#else
void talk_to_pertochek(pmessage)
     MESSAGE *pmessage;
#endif /* ANSI */
#endif /* PERTO_NO_SERVER_DAEMON */
{
   char temp[32];
   int tam;
	int timeout;

#ifndef PERTO_NO_SERVER_DAEMON
	int dtroff;
	static MESSAGE message;
	MESSAGE *pmessage;

	pmessage = &message;

	dtroff = (srvcfg->pchekcaps & PCHEK_CAPS_DTROFF);

   if (inic_stxetx(srvcfg->device, srvcfg->baud, dtroff) == FALSE) {
   	fprintf(stderr, "ERRO: comunicacoes [1:%s]\n", srvcfg->name);
      exit(1);
   }

   if (dtroff) {
		/* The previos inic_stxetx() was for config testing only. */
   	stop_stxetx();
	}

   while (1) {
      if (! receive(srvcfg->msgkey, pmessage, sizeof(*pmessage)))
			continue;

      if (dtroff)
         if (inic_stxetx(srvcfg->device, srvcfg->baud, dtroff) == FALSE) {
            fprintf(stderr, "ERRO: comunicacoes [2:%s]\n", srvcfg->name);
      		pmessage->estado = P_ERRO_INIT_SERIAL;
      		send(pmessage->chave_cliente, pmessage, sizeof(*pmessage));
            exit(1);
         }

#endif /* PERTO_NO_SERVER_DAEMON */

      switch (pmessage->dados[0]) {
         case P_IMP_FOLHA:
         case P_IMP_FOLHA_PARCIAL:
            comando1(pmessage, pmessage->tam, TMOUT_COMANDO2);
            break;
         case P_BOM_PARA:
         case P_CAMPO_EXTRA:
            comando1(pmessage, sizeof(struct p_cmd18), TMOUT_COMANDO1);
            break;
         case P_PRG_DATA_CHK:
            comando1(pmessage, sizeof(struct p_cmd4), TMOUT_COMANDO1);
            break;
         case P_IMP_VERSO:
            ((struct p_cmd11 *)(pmessage->dados))->f1 = 0xff;
            ((struct p_cmd11 *)(pmessage->dados))->f2 = 0xff;
            comando1(pmessage, sizeof(struct p_cmd11), TMOUT_COMANDO2);
            break;
         case P_PRG_CIDADE:
            comando1(pmessage, sizeof(struct p_cmd2), TMOUT_COMANDO1);
            break;
         case P_IMP_COMPLETO:
         case P_IMP_PARCIAL:
            comando1(pmessage, sizeof(struct p_cmd1), TMOUT_COMANDO2);
            break;
         case P_PRG_BENEFIC:
            comando1(pmessage, sizeof(struct p_cmd3), TMOUT_COMANDO1);
            break;
         case P_IMP_COPIA:
            ((struct p_cmd7 *)(pmessage->dados))->f1 = 0xff;
            ((struct p_cmd7 *)(pmessage->dados))->f2 = 0xff;
            ((struct p_cmd7 *)(pmessage->dados))->f3 = 0xff;
            /*((struct p_cmd7 *)(pmessage->dados))->f4 = 0xff;*/
            comando1(pmessage, sizeof(struct p_cmd7), TMOUT_COMANDO2);
            break;
         case P_ENVIA_CONF:
            comando2(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO1);
            break;
         case P_ENVIA_BENEF:
            comando2(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO1);
            break;
         case P_ENVIA_CIDADE:
            comando2(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO1);
            break;
#ifndef PERTO_NOT_STANDARD
         case P_ALINHA_LE_ENVIA:
#endif /* PERTO_NOT_STANDARD */
         case P_ALINHA_E_LE:
            comando3(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO2);
            break;
         case P_EXPULSA_CHEQUE:
            comando1(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO2);
            break;
         case P_CAD_CHEQUES:
            if (pmessage->dados[1] == '0')
               comando5(pmessage, 5, TMOUT_COMANDO1);
            else if (pmessage->dados[1] == '1')
               comando5(pmessage, sizeof(struct p_cmd10), TMOUT_COMANDO1);
            else
               comando4(pmessage, 5, TMOUT_COMANDO1);
            break;
         case P_PRG_MODO:
            comando1(pmessage, sizeof(struct p_cmd6), TMOUT_COMANDO1);
            break;
         case P_PRG_COLUNAS:
            comando1(pmessage, sizeof(struct p_cmd14), TMOUT_COMANDO1);
            break;
         case P_PRG_MOEDA:
            tam = acerta_tam_moeda(pmessage);
            comando1(pmessage, tam + 1, TMOUT_COMANDO1);
            break;
         case P_PRG_TELEFONE:
            ((struct p_cmd12 *)(pmessage->dados))->f1 = 0xff;
            comando1(pmessage, sizeof(struct p_cmd12), TMOUT_COMANDO1);
            break;
         case P_ENVIA_VERSAO:
            comando2(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO1);
            break;
         case P_PRG_CHEQUE_ADM:
            comando1(pmessage, sizeof(struct p_cmd15), TMOUT_COMANDO1);
            break;
         case P_IMP_CHEQUE_ADM:
            comando1(pmessage, sizeof(struct p_cmd1), TMOUT_COMANDO2);
            break;
         case P_PRG_MODO2:
            comando1(pmessage, sizeof(struct p_cmd20), TMOUT_COMANDO1);
            break;
         case P_LE_CARTAO:
            comando3(pmessage, pmessage->tam, TMOUT_COMANDO2);
            break;
         case P_ENVIA_COM_IMP:
            comando1(pmessage, sizeof(struct p_cmd22), TMOUT_COMANDO1);
            break;
         case P_ESPERA_TEC_PINPAD:
            memcpy(temp, &pmessage->dados[1], 3);
            temp[3] = 0;
            timeout = atoi(temp);
            comando3(pmessage, sizeof(struct p_cmd23), 
                  timeout + TMOUT_COMANDO1);
            break;
         case P_ENVIA_COM_LCD:
            comando1(pmessage, pmessage->tam, TMOUT_COMANDO1);
            break;
         case P_ENVIA_COM_PINPAD:
            pmessage->dados[pmessage->tam] = 0;
            memcpy(temp, &pmessage->dados[1], 3);
            temp[3] = 0;
            timeout = atoi(temp);
            comando3(pmessage, pmessage->tam, timeout + TMOUT_COMANDO1);
            break;
         case P_PRG_CHAVE_CRIPTO:
            comando1(pmessage, sizeof(struct p_cmd26), TMOUT_COMANDO1);
            break;
         case P_ENVIA_ULTIMO_CMC7:
            comando3(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO2);
            break;
         case P_LISTA_NEGRA:
            comando5(pmessage, sizeof(struct p_cmd27), TMOUT_COMANDO1);
            break;
         case P_IMP_COMPLETO_CHQ_TB:
         case P_IMP_PARCIAL_CHQ_TB:
            comando1(pmessage, sizeof(struct p_cmd28), TMOUT_COMANDO2);
            break;
            
         case P_ESP_TECLA_PCHEK:
            memcpy(temp, &pmessage->dados[1], 3);
            temp[3] = 0;
            timeout = atoi(temp);
            comando3(pmessage, sizeof(struct p_cmd23), 
                  timeout + TMOUT_COMANDO1);
            break;

         case P_ESPERA_MOSTRA_TECLA:
            memcpy(temp, &pmessage->dados[5], 3);
            temp[3] = 0;
            timeout = atoi(temp);
            comando3(pmessage, sizeof(struct p_cmd29), 
                  timeout + TMOUT_COMANDO1);
            break;

         case P_ALINHA_LE_DOC_BARRAS:
            comando3(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO2);
            break;

         case P_ENVIA_STATUS_EQUIP:
            comando2(pmessage, sizeof(struct p_cmd0), TMOUT_COMANDO1);
            break;
      }

#ifndef PERTO_NO_SERVER_DAEMON
      if (dtroff)
         stop_stxetx();

      send(pmessage->chave_cliente, pmessage, sizeof(*pmessage));
   }
#endif /* PERTO_NO_SERVER_DAEMON */
}

#ifdef ANSI
static void comando1(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando1(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[32];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE) {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 32;

   memset(temp, ' ', sizeof(temp));

   if (rec_stxetx(temp, &tam, timeout) == FALSE) {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   temp[4] = 0;

   m_ptr->estado = trata_erro(&temp[1]);
}

#ifdef ANSI
static void comando2(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando2(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[256];
   char erro[4];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE) {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 256;

   memset(temp, ' ', sizeof(temp));

   if (rec_stxetx(temp, &tam, timeout) == FALSE) {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   memcpy(m_ptr->dados, temp, tam);
   m_ptr->tam = tam;
   m_ptr->estado = P_TERMINO_OK;
}

#ifdef ANSI
static void comando3(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando3(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[TAMBUF];
   char erro[4];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE) {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 256;

   memset(temp, ' ', sizeof(temp));
   if (rec_stxetx(temp, &tam, timeout) == FALSE) {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   memcpy(m_ptr->dados, temp, tam);
   m_ptr->tam = tam;

   memcpy(erro, temp + 1, 3);
   erro[3] = 0;

   m_ptr->estado = trata_erro(erro);
}

#ifdef ANSI
static void comando4(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando4(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[TAMBUF];
   char erro[4];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE) {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 256;

   memset(temp, ' ', sizeof(temp));

   if (rec_stxetx(temp, &tam, timeout) == FALSE) {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   if (tam == 5) {
      memcpy(erro, &temp[2], 3);
      erro[3] = 0;
      m_ptr->estado = trata_erro(&erro[0]);
   }
   else {
      memcpy(m_ptr->dados, temp, tam);
      m_ptr->tam = tam;
      m_ptr->estado = P_TERMINO_OK;
   }
}

#ifdef ANSI
static void comando5(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando5(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[32];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE) {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 32;

   memset(temp, ' ', sizeof(temp));

   if (rec_stxetx(temp, &tam, timeout) == FALSE) {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   temp[5] = 0;

   m_ptr->estado = trata_erro(&temp[2]);
}

#ifdef ANSI
static void comando6(MESSAGE * m_ptr, int tamanho, int timeout)
#else
static void comando6(m_ptr, tamanho, timeout)
     MESSAGE *m_ptr;
     int tamanho;
     int timeout;
#endif
{
   int tam;
   char temp[256];
   char erro[4];

   dump((char *)m_ptr, "tx", tamanho);

   if (trans_stxetx(m_ptr->dados, tamanho, TMOUT_TX) == FALSE)
   {
      m_ptr->estado = P_ERRO_TX;
      return;
   }

   tam = 256;

   memset(temp, ' ', sizeof(temp));

   if (rec_stxetx(temp, &tam, timeout) == FALSE)
   {
      m_ptr->estado = P_ERRO_RX;
      return;
   }

   dump(temp, "rx", tam);

   if (tam == 4)
   {
      memcpy(erro, &temp[2], 3);
      erro[3] = 0;
      m_ptr->estado = trata_erro(&erro[0]);
   }
   else
   {
      memcpy(m_ptr->dados, temp, tam);
      m_ptr->tam = tam;
      m_ptr->estado = P_TERMINO_OK;
   }
}


#ifdef ANSI
static int trata_erro(char *erro_str)
#else
static int trata_erro(erro_str)
     char *erro_str;
#endif
{
   int erro;

   erro = atoi(erro_str);

   switch (erro) {
      case P_TERMINO_OK:
      case P_ERRO_DADOS:
      case P_ERRO_TAMANHO:
      case P_ERRO_CMC_INV:
      case P_ERRO_MOTOR1:
      case P_ERRO_JA_MARCADO:
      case P_ERRO_MOTOR2:
      case P_ERRO_BANCO:
      case P_ERRO_FITA:
      case P_ERRO_SENSOR1:
      case P_ERRO_SENSOR2:
      case P_ERRO_SENSOR3:
      case P_ERRO_CABECA:
      case P_ERRO_DIG_VER:
      case P_ERRO_SEM_CAR_MAG:
      case P_ERRO_SEM_DOC:
      case P_ERRO_INSERCAO:

      case P_ERRO_ALINHA:
      case P_ERRO_ALINHA_S1:
      case P_ERRO_ALINHA_S2:
      case P_ERRO_ALINHA_S12:

      case P_ERRO_PREENCH:
      case P_ERRO_PREENCH_S1:
      case P_ERRO_PREENCH_S2:
      case P_ERRO_PREENCH_S12:

      case P_ERRO_CARAC_INEXIST:

      case P_ERRO_LISTA_CHEIA:
      case P_ERRO_RELOGIO:

      case P_ERRO_CHQ_AUSENTE:
      case P_ERRO_PIN_TECL_AUSENTE:
      case P_ERRO_TRANSMISSAO:
      case P_ERRO_TX_IMPRESSORA:
      case P_ERRO_PINPAD:

      case P_ERRO_CHQ_NA_LISTA:
      case P_ERRO_CHQ_FORA_LISTA:
      case P_ERRO_CANCELADO:
      case P_ERRO_CAD_CHEIO:
      case P_ERRO_CAD:

      case P_ERRO_LEITURA_CARTAO:
      case P_ERRO_LEITURA_TRILHA1:
      case P_ERRO_LEITURA_TRILHA2:
      case P_ERRO_LEITURA_TRILHA3:

      case P_ERRO_CMC:

      case P_ERRO_EOT:
      case P_ERRO_NAK:
      case P_ERRO_SEM_RESPOSTA:
      case P_ERRO_TEMPO_ESGOTADO:
         return (erro);
      default:
         return (P_ERRO_INVALIDO);
   }
}

#ifdef ANSI
static int acerta_tam_moeda(MESSAGE * p)
#else
static int acerta_tam_moeda(p)
     MESSAGE *p;
#endif
{
   char *ptr;
   int tam;
   int umbranco;

   ptr = ((struct p_cmd8 *)(p->dados))->moeda;
   tam = 0;
   umbranco = 0;
   while (tam < 20)
   {
      if (*ptr == ' ')
      {
         if (umbranco == 1)
            break;
         else
            umbranco = 1;
      }
      else
      {
         if (umbranco == 1)
         {
            umbranco = 0;
            tam++;
         }
         tam++;
      }
      ptr++;
   }
   return (tam);
}

#ifdef DEBUG
#ifdef ANSI
void dump(char *ptr, char *tipo, int tam)
#else
int dump(ptr, tipo, tam)
     char *ptr;
     char *tipo;
     int tam;
#endif
{
   unsigned char *p;
   char tmp[TAMBUF];
   int i;
   int j;

   p = (unsigned char *)ptr;

   if (strcmp(tipo, "tx") == 0)
      p += 8;

   memset(tmp, 0, TAMBUF);
   i = 0;
   j = tam;
   while (j--)
   {
      if (*p == 0xff)
         tmp[i] = '^';
      else if (*p < ' ' || *p > 0x7f)
         tmp[i] = '?';
      else
         tmp[i] = *p;
      p++;
      i++;
   }
   fprintf(stderr, "%s: [%s] %d\n", tipo, tmp, tam);
}
#endif /* DEBUG */

/* server.c eof ------------------------------------------------------------ */
