/* 18/12/96 09:55:16
 ******************************************************************************
 **                                                                           *
 **  MODULO : stxetx.c                                                        *
 **                                                                           *
 **  FUNCAO : comunicacao com a impressora PERTO CHECK                        *
 **                                                                           *
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  Registro de Alteracoes/Versoes :                                         *
 **  ==============================                                           *
 **                                                                           *
 **  VERSAO   DATA    RESPONSAVEL   DESCRICAO                                 *
 **                                                                           *
 **  V1.00  13/06/96  Magoga        Criacao do arquivo                        *
 **  V1.01  2003/01/24 Tarcisio Kreutz, Perto
 **     - Disable DTR (ioctl(TIOCMBIC)) for compatibility with new Pertochek
 **  serial device (when DIP switch 4 is ON, this new machine just "forwards" 
 **  data to the PINPAD if DTR is ON. For the Pertochek to process the data, 
 **  the DTR must be OFF).
 **     - Better error message detail. 
 **       (Error info should receive additional improvements!).
 **  V1.02  2004/02/09 Tarcisio Kreutz, Perto
 **     - Disable RTS, not DTR, if compiled for Qualita.
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  COMENTARIOS  :                                                           *
 **                                                                           *
 **  ATENCAO :                                                                *
 **  =======                                                                  *
 **                                                                           *
 ******************************************************************************
 */

/*#######################  I N C L U D E S  ##################################*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <termio.h>
#include <time.h>
#include <errno.h>
extern int errno;

#include "perto.h"

/*#######################  C O N S T A N T E S  ##############################*/

#define NUM_TENTATIVAS  3

#define STX             2
#define ETX             3
#define ACK             6
#define NAK          0x15

/*#######################  P R O T O T I P O S  ##############################*/

#ifdef ANSI
static int getC(unsigned char *);
static int putC(char);
static void flushC(void);
static int liga_timer(int);
static int fim_timer(void);
#endif

/*#######################  V A R I A V E I S  ################################*/

/*=======================  L o c a i s  ======================================*/

static int f_desc;
static time_t fim_tempo;

/*=======================  P u b l i c a s  ==================================*/

/*=======================  E x t e r n a s  ==================================*/

/*#######################  R O T I N A S  ####################################*/

#ifdef ANSI
int rec_stxetx(char *buf, int *tam, int timeout)
#else
int rec_stxetx(buf, tam, timeout)
     char *buf;
     int *tam;
     int timeout;
#endif
{
   int i;
   int tentativ;
   int fim_buf;
   unsigned char c;
   unsigned char bcc;
   unsigned char temp[TAMBUF];

   tentativ = NUM_TENTATIVAS;

   if (*tam > TAMBUF || *tam <= 0)
      return (FALSE);

   while (tentativ--)
   {

      if (timeout != 0)
         liga_timer(timeout);

      do
      {
         if (getC(&c) == TRUE)
            if (c == STX)
               break;

         if (timeout != 0)
            if (fim_timer() == TRUE)
               return (FALSE);
      } while (1);

      i = 0;

      bcc = STX;

      fim_buf = FALSE;

      if (timeout != 0)
         liga_timer(timeout);

      do
      {
         if (i == *tam)
            return (FALSE);

         if (getC(&c) == TRUE)
         {
            if (i < TAMBUF)
               temp[i] = c;
            else
               fim_buf = TRUE;

            bcc ^= c;

            if (c == ETX)
               break;

            i++;
         }

         if (timeout != 0)
            if (fim_timer() == TRUE)
               return (FALSE);
      } while (1);

      if (timeout != 0)
         liga_timer(timeout);

      do
      {
         if (getC(&c) == TRUE)
            break;

         if (fim_timer() == TRUE)
            return (FALSE);
      } while (1);

      if (bcc != c || fim_buf == TRUE)
      {
         if (putC(NAK) == FALSE)
            return (FALSE);
      }
      else
      {
         if (putC(ACK) == FALSE)
            return (FALSE);
         else
            break;
      }
   }

   *tam = i;
   memcpy(buf, (char *)temp, i);
   return (TRUE);
}

#ifdef ANSI
int trans_stxetx(char *buf, int tam, int timeout)
#else
int trans_stxetx(buf, tam, timeout)
     char *buf;
     int tam;
     int timeout;
#endif
{
   unsigned char bcc;
   unsigned char c;
   int i;
   int tentativ;

   if (tam > TAMBUF || tam <= 0)
      return (FALSE);

   tentativ = NUM_TENTATIVAS;

   while (tentativ--)
   {

      i = 0;

      flushC();

      if (putC(STX) == FALSE)
         return (FALSE);

      bcc = STX;

      if (timeout != 0)
         liga_timer(timeout);

      do
      {
         c = (unsigned char)buf[i++];

         if (putC(c) == FALSE)
            return (FALSE);

         bcc ^= c;

         if (i == tam)
            break;

         if (timeout != 0)
            if (fim_timer() == TRUE)
               return (FALSE);
      } while (1);

      if (putC(ETX) == FALSE)
         return (FALSE);

      bcc ^= ETX;

      if (putC(bcc) == FALSE)
         return (FALSE);

      if (timeout != 0)
         liga_timer(timeout);

      do
      {
         if (getC(&c) == TRUE)
         {
            if (c == ACK)
               return (TRUE);
            else
               break;
         }

         if (timeout != 0)
            if (fim_timer() == TRUE)
               return (FALSE);
      } while (1);
   }
}

#ifdef ANSI
int inic_stxetx(char *dev, int baud, int dtroff)
#else
int inic_stxetx(dev, baud, dtroff)
     char *dev;
     int baud;
     int dtroff;
#endif
{
   struct termio tbuf;

   if ((f_desc = open(dev, O_RDWR)) == -1)
   {
#ifndef CLIENT_NO_MESSAGES
      fprintf(stderr, "ERRO: Falha abertura porta serial. [%s: %s]\n",
              dev, strerror(errno));
#endif /* CLIENT_NO_MESSAGES */
      return (FALSE);
   }

   memset((char *)&tbuf, 0, sizeof(struct termio));

   tbuf.c_iflag = IGNBRK;
   tbuf.c_oflag = 0;
   tbuf.c_cflag = baud | CS8 | CREAD | CLOCAL | HUPCL;
   tbuf.c_lflag = 0;
   tbuf.c_cc[VMIN] = 0;
   tbuf.c_cc[VTIME] = 2;        /* 2 * 0.1s */

   if (ioctl(f_desc, TCSETAF, &tbuf) == -1)
   {
#ifndef CLIENT_NO_MESSAGES
      fprintf(stderr, "ERRO: Falha ao configurar porta serial [%s]. "
              "[ioctl(TCSETAF:%u): %s]\n", dev,
              baud == B1200 ? 1200 :
              baud == B2400 ? 2400 :
              baud == B4800 ? 4800 :
              baud == B9600 ? 9600 : ~0, strerror(errno));
#endif /* CLIENT_NO_MESSAGES */
      close(f_desc);
      f_desc = -1;
      return (FALSE);
   }

   if (dtroff)
   {
#ifndef PERTO_NOT_STANDARD
      /* Disable DTR. */
      dtroff = TIOCM_DTR;
#else
      /* Disable RTS. (Qualita) */
      dtroff = TIOCM_RTS;
#endif /* PERTO_NOT_STANDARD */

      if (ioctl(f_desc, TIOCMBIC, &dtroff) == -1)
      {
#ifndef CLIENT_NO_MESSAGES
         fprintf(stderr, "ERRO: Falha desativando DTR da porta serial [%s]. "
                 "[ioctl(TIOCMBIC:"
#ifndef PERTO_NOT_STANDARD
					  "TIOCM_DTR"
#else
					  "TIOCM_RTS"
#endif /* PERTO_NOT_STANDARD */
					  "): %s]\n", 
					  dev, strerror(errno));
#endif /* CLIENT_NO_MESSAGES */
         close(f_desc);
         f_desc = -1;
         return (FALSE);
      }
   }

   return (TRUE);
}                               /* inic_stxetx() */

#ifdef ANSI
int stop_stxetx(void)
#else
int stop_stxetx()
#endif
{
   if (f_desc >= 0)
      close(f_desc);
   f_desc = -1;
}                               /*stop_stxetx() */

#ifdef ANSI
static int getC(unsigned char *c)
#else
int getC(c)
     unsigned char *c;
#endif
{
   char tmp;
   int ret;

   ret = read(f_desc, &tmp, 1);

   if (ret == 0 || ret == -1)
      return (FALSE);
   else
   {
      *c = tmp;
      return (TRUE);
   }
}

#ifdef ANSI
static int putC(char c)
#else
int putC(c)
     char c;
#endif
{
   int ret;

   ret = write(f_desc, &c, 1);

   if (ret == -1)
      return (FALSE);
   else
      return (TRUE);
}

#ifdef ANSI
static void flushC(void)
#else
int flushC()
#endif
{
   ioctl(f_desc, TCFLSH, 0);
}

#ifdef ANSI
static int liga_timer(int timeout)
#else
int liga_timer(timeout)
     int timeout;
#endif
{
   fim_tempo = time(NULL) + timeout;
}

#ifdef ANSI
static fim_timer(void)
#else
int fim_timer()
#endif
{
   if (time(NULL) > fim_tempo)
      return (TRUE);
   else
      return (FALSE);
}

/* stxetx.c eof ------------------------------------------------------------- */
